/*
* Copyright (c) 2016 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import Foundation

enum Language: String {
  case English = "en"
  case Spanish = "es"
  
  init(index: Int) {
    if index == 0 {
      self = .English
    } else {
      self = .Spanish
    }
  }
}

struct WikiService {
  fileprivate typealias JSONDictionary = [String: AnyObject]
  fileprivate static let baseURL = "wikipedia.org/w/api.php"
  
  static func getWikiForSearchString(_ string: String, language: Language, completion: @escaping (WikiPage?) -> ()) {
    getTitleForSearchString(string, language: language) { (title: String?) -> () in
      guard let title = title else {
        mainThread({ completion(nil) })
        return
      }
      
      getWikiDetailsForTitle(title, language: language) { (wikiPage: WikiPage?) -> () in
        mainThread({ completion(wikiPage) })
      }
    }
  }
  
  static func getAlternateVersionOfPage(_ page: WikiPage, completion: @escaping (WikiPage?) -> ()) {
    let newLanguage = page.language == .English ? Language.Spanish : Language.English
    getAlternateTitleFromPage(page, toLanguage: newLanguage) { (title: String?) -> () in
      guard let title = title else {
        mainThread({ completion(nil) })
        return
      }
      
      getWikiDetailsForTitle(title, language: newLanguage) { (wikiPage: WikiPage?) -> () in
        mainThread({ completion(wikiPage) })
      }
    }
  }
}

// MARK: - Private Methods
private extension WikiService {
  static func mainThread(_ completion: @escaping () -> ()) {
    DispatchQueue.main.async {
      completion()
    }
  }
  
  static func getTitleForSearchString(_ string: String, language: Language, completion: @escaping (String?) -> ()) {
    let escapedString = "https://\(language.rawValue).\(baseURL)?action=query&list=search&format=json&srlimit=1&srprop=sectiontitle&srsearch=\(string)".addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
    let url = URL(string: escapedString)!

    URLSession.shared.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data, error == nil else {
        completion(nil)
        return
      }

      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject
      guard let query = json?["query"] as? JSONDictionary else {
        mainThread({ completion(nil) })
        return
      }
      
      if let searchInfo = query["searchinfo"] as? JSONDictionary, let suggestion = searchInfo["suggestion"] as? String {
        getTitleForSearchString(suggestion, language: language, completion: completion)
      } else {
        guard let search = query["search"] as? [JSONDictionary], let title = search.first?["title"] as? String else {
          mainThread({ completion(nil) })
          return
        }
        
        mainThread({ completion(title) })
      }
      }.resume()
  }
  
  static func getAlternateTitleFromPage(_ page: WikiPage, toLanguage: Language, completion: @escaping (String?) -> ()) {
    let escapedString = "https://\(page.language.rawValue).\(baseURL)?action=query&format=json&prop=langlinks&lllimit=1&lllang=\(toLanguage.rawValue)&titles=\(page.title)".addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
    let url = URL(string: escapedString)!
    
    print("URL: \(url)")
    
    URLSession.shared.dataTask(with: url, completionHandler: { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data else {
        mainThread({ completion(nil) })
        return
      }
      
      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject
      
      guard let query = json?["query"] as? JSONDictionary, let pages = query["pages"] as? [String: JSONDictionary], let (_, page) = pages.first, let langLinks = page["langlinks"] as? [JSONDictionary], let spanishLang = langLinks.first, let translatedTitle = spanishLang["*"] as? String else {
        mainThread({ completion(nil) })
        return
      }
      
      completion(translatedTitle)
    }).resume()
  }
  
  static func getWikiDetailsForTitle(_ title: String, language: Language = .English, completion: @escaping (WikiPage?) -> ()) {
    let escapedString = "https://\(language.rawValue).\(baseURL)?action=query&format=json&prop=extracts|pageimages&exintro&explaintext&pithumbsize=800&exsentences=8&titles=\(title)".addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
    let url = URL(string: escapedString)!
    
    URLSession.shared.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data else {
        mainThread({ completion(nil) })
        return
      }
      
      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject
      guard let query = json?["query"] as? JSONDictionary, let pages = query["pages"] as? [String: JSONDictionary], let (_, page) = pages.first, let title = page["title"] as? String, let extract = page["extract"] as? String else {
        mainThread({ completion(nil) })
        return
      }
      
      var thumbnailURL: String? = nil
      if let thumnailInfo = page["thumbnail"] as? JSONDictionary, let sourceURL = thumnailInfo["source"] as? String {
        thumbnailURL = sourceURL
      }
      
      let wikiPage = WikiPage(title: title, extract: extract, imageURLString: thumbnailURL, language: language)
      mainThread({ completion(wikiPage) })
      }.resume()
  }
}
