/*
* Copyright (c) 2016 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import UIKit

enum CategoryType: Int, CustomStringConvertible {
  case bestPicture
  case animatedFeature
  
  var description: String {
    switch self {
    case .bestPicture: return "Best Picture"
    case .animatedFeature: return "Animated Feature"
    }
  }
}

class MovieService {
  static let sharedInstance = MovieService()
  let session: URLSession
  let URLCache = Foundation.URLCache(memoryCapacity: 20 * 1024 * 1024, diskCapacity: 100 * 1024 * 1024, diskPath: "ImageDownloadCache")
  
  init() {
    let config = URLSessionConfiguration.default
    config.requestCachePolicy = NSURLRequest.CachePolicy.returnCacheDataElseLoad
    config.urlCache = URLCache
    session = URLSession(configuration: config)
  }
  
  func getMovieNominationsForCategory(_ category: CategoryType, completion: @escaping ([Movie]) -> Void) {
    var movies: [Movie] = []
    
    // Simulating an API call using a prebundled .plist file
    if let plistPath = Bundle.main.path(forResource: "BestPictureNominees", ofType: "plist"), let dict = NSDictionary(contentsOfFile: plistPath) as? Dictionary<String, [[String: AnyObject]]> {
      
      if let categoryData = dict[category.description] {
        for movieInfo in categoryData {
          if let movie = Movie(dictionary: movieInfo) {
            movies.append(movie)
          }
        }
      }
    }
    
    DispatchQueue.main.async {
      completion(movies)
    }
  }
  
  func getSampleMovie(_ completion: @escaping (Movie?) -> Void) {
    getMovieNominationsForCategory(.bestPicture) { movies in
      completion(movies.first)
    }
  }
  
  func getImageFromMovie(_ movie: Movie, completion: @escaping (UIImage?) -> Void) {
    if let url = URL(string: movie.imageURL) {
      let urlRequest = URLRequest(url: url, cachePolicy: NSURLRequest.CachePolicy.returnCacheDataElseLoad, timeoutInterval: 30.0)
      
      if let response = URLCache.cachedResponse(for: urlRequest) {
        let image = UIImage(data: response.data)
        DispatchQueue.main.async {
          completion(image)
          return
        }
      }
      
      session.dataTask(with: urlRequest) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
        guard let response = response, let data = data else {
          completion(nil)
          return
        }
        
        MovieService.sharedInstance.URLCache.storeCachedResponse(CachedURLResponse(response:response, data:data, userInfo:nil, storagePolicy:.allowed), for: urlRequest) // commit response into cache
        
        let downloadedImage = UIImage(data: data)
        DispatchQueue.main.async {
          completion(downloadedImage)
        }
      }.resume()
    }
  }
  
  static func setMovie(_ movie: Movie, asWatched watched: Bool) {
    UserDefaults.standard.set(watched, forKey: movie.title)
    UserDefaults.standard.synchronize()
  }
  
  static func isMovieWatched(_ movie: Movie) -> Bool {
    return UserDefaults.standard.bool(forKey: movie.title)
  }
}

extension UIImageView {
  func setImageFromMovie(_ movie: Movie) {
    image = nil
    MovieService.sharedInstance.getImageFromMovie(movie) { image in
      if self.image == nil {
        self.image = image
      }
    }
  }
}
