/*
* Copyright (c) 2016 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import UIKit

class MovieCollectionViewController: UICollectionViewController {
  
  fileprivate let reuseIdentifier = "MovieCell"
  fileprivate let detailSegueIdentifier = "DetailSegue"

  var movies = [Movie]()
  var movieOrder: SortOrder = .alphabetical

  var movieCategory: CategoryType {
    if let tabBarController = tabBarController,
      let splitViewController = splitViewController,
      let index = tabBarController.viewControllers?.index(of:
        splitViewController) {
      return CategoryType(rawValue: index)!
    }
    return .bestPicture
  }

  override func viewDidLoad() {
    super.viewDidLoad()
    
    MovieService.sharedInstance.getMovieNominationsForCategory(movieCategory) { movies in
      self.movies = movies
      self.collectionView?.reloadData()
    }
  }

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    collectionView?.reloadData()
  }

  // MARK: - Navigation
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    // 1, 2
    if let destinationViewController =
      segue.destination as?
      MovieDetailViewController, let selectedIndex =
      collectionView?.indexPathsForSelectedItems?.first {
      // 3
      let sectionMovies = moviesForSection(selectedIndex.section)
      destinationViewController.movie =
        sectionMovies[selectedIndex.item]
    }
  }
}

// MARK: UICollectionViewDataSource
extension MovieCollectionViewController {
  override func numberOfSections(in collectionView: UICollectionView) -> Int {
    return 2
  }

  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return moviesForSection(section).count
  }

  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    // 1
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: reuseIdentifier, for: indexPath) as! MovieCollectionViewCell
    
    // 2
    let sectionMovies = moviesForSection(indexPath.section)
    let movie = sectionMovies[indexPath.item]
    
    // 3
    cell.movieTitleLabel.text = movie.title
    cell.movieImageView.setImageFromMovie(movie)
    return cell
  }
  
  override func collectionView(_ collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, at indexPath: IndexPath) -> UICollectionReusableView {
    // 1
    switch kind {
      // 2
    case UICollectionElementKindSectionHeader:
      // 3
      let headerView = collectionView.dequeueReusableSupplementaryView(ofKind: kind, withReuseIdentifier: "MovieHeaderView", for: indexPath) as! MovieCollectionReusableView
      
      // 4
      if indexPath.section == 0 {
        headerView.headerLabel.text = "Unwatched Movies"
      } else {
        headerView.headerLabel.text = "Watched Movies"
      }
      
      return headerView
    default:
      // 5
      assert(false, "Unexpected Supplementary View Type")
    }
  }
}

// MARK: - UICollectionViewDelegateFlowLayout
extension MovieCollectionViewController: UICollectionViewDelegateFlowLayout {
  // 1
  func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
    return CGSize(width: 300, height: 525)
  }
  
  // 3
  func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAt section: Int) -> UIEdgeInsets {
    return UIEdgeInsets(top: 0, left: 90, bottom: 70, right: 90)
  }
}

// MARK: UICollectionViewDelegate
extension MovieCollectionViewController {
  override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    performSegue(withIdentifier: detailSegueIdentifier,
                 sender: nil)
  }
  
  // 1
  override func collectionView(_ collectionView: UICollectionView, willDisplay cell: UICollectionViewCell, forItemAt indexPath: IndexPath) {
    // 2
    cell.alpha = 0.0
    // 3
    UIView.animate(withDuration: 1.0) {
      cell.alpha = 1.0
    }
  }
  
  // 1
  override func collectionView(_ collectionView: UICollectionView, willDisplaySupplementaryView view: UICollectionReusableView, forElementKind elementKind: String, at indexPath: IndexPath) {
    // 2
    view.frame.origin.x += 50
    view.alpha = 0.0
    
    // 3
    UIView.animate(withDuration: 1.0) {
      view.frame.origin.x -= 50
      view.alpha = 1.0
    }
  }
}

// MARK: Private Methods
private extension MovieCollectionViewController {
  func moviesForSection(_ section: Int) -> [Movie] {
    if section == 0 {
      return sortMovies(movies.filter({
        MovieService.isMovieWatched($0) == false }))
    } else {
      return sortMovies(movies.filter({
        MovieService.isMovieWatched($0) == true }))
    }
  }

  func sortMovies(_ movies: [Movie]) -> [Movie] {
    switch movieOrder {
    case .alphabetical:
      return movies.sorted{$0.title < $1.title}
    case .userRating:
      return movies.sorted{$0.userRating > $1.userRating}
    case .length:
      return movies.sorted{$0.runtime > $1.runtime}
    }
  }
}

// MARK: MovieSortDelegate
extension MovieCollectionViewController: MovieSortDelegate {
  func orderChanged(_ order: SortOrder) {
    movieOrder = order
    collectionView?.reloadData()
  }
}
