/*
* Copyright (c) 2016 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import UIKit

class ViewController: UIViewController {
  @IBOutlet weak var cityTextField: UITextField!
  @IBOutlet weak var countryTextField: UITextField!
  @IBOutlet weak var mapView: UIImageView!
  @IBOutlet weak var collectionView: UICollectionView!
  @IBOutlet weak var tableView: UITableView!
  @IBOutlet weak var logoView: UIImageView!
  
  var updateTimer: Timer?

  // Data
  var astronauts = [String]() { didSet { collectionView.reloadData() } }
  var flyByTimes = [[FlyBy]]() { didSet { tableView.reloadData() } }

  // Map Markers
  lazy var stationMarker: UIImageView = {
    let markerView = UIImageView(image: UIImage(named: "ISS"))
    let aspectRatio = markerView.frame.width / markerView.frame.height
    markerView.frame = CGRect(x: 0, y: 0, width: 100, height: 100/aspectRatio)
    markerView.contentMode = .scaleAspectFit
    markerView.isHidden = true
    self.mapView.addSubview(markerView)
    return markerView
  }()

  lazy var cityMarker: UIView = {
    let markerView = UIView(frame: CGRect(x: 0, y: 0, width: 14, height: 14))
    markerView.backgroundColor = .red
    markerView.isHidden = true
    self.mapView.addSubview(markerView)
    return markerView
  }()

  override func viewDidLoad() {
    super.viewDidLoad()

    updateCurrentISSLocation()
    updateTimer = Timer.scheduledTimer(timeInterval: 10.0, target: self, selector: #selector(updateCurrentISSLocation), userInfo: nil, repeats: true)

    updateAstronautsInISS()
    updateFlyByTimes()
  }
}

// MARK: Actions / Selectors
extension ViewController {
  @IBAction func updateFlyByTimes() {
    guard let city = cityTextField.text, let country = countryTextField.text else {
      // Notify User
      return
    }

    SpaceService.getLatLongForCity(city, country: country) { error, lat, lng in
      if let _ = error {
        let alert = UIAlertController(title: "Oops", message: "There was a problem getting fly by times for your location.", preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "OK", style: .cancel, handler: nil))
        self.present(alert, animated: true, completion: nil)
      } else {
        self.cityMarker.center = self.getMapCoordinateFromLocation(Location(lat: lat, lng: lng))
        self.cityMarker.isHidden = false

        SpaceService.getGroupedFlyByTimes(lat, lng: lng) { flyBys in
          self.flyByTimes = flyBys
        }
      }
    }
  }

  func updateCurrentISSLocation() {
    SpaceService.getCurrentISSLocation { (location: Location?) -> Void in
      if let location = location {
        self.stationMarker.center = self.getMapCoordinateFromLocation(location)
        self.stationMarker.isHidden = false
      }
    }
  }
}

// MARK: Private methods
private extension ViewController {
  func updateAstronautsInISS() {
    SpaceService.getAstronautsInISS { (astronauts: [String]) -> Void in
      self.astronauts = astronauts
    }
  }

  func getMapCoordinateFromLocation(_ location: Location) -> CGPoint {
    let mapWidth = Double(mapView.frame.width)
    let mapHeight = Double(mapView.frame.height)

    let latRad = location.lat*M_PI/180.0
    let mercN = log(tan(M_PI_4 + latRad/2.0))

    let x = (location.lng+180) * (mapWidth/360.0)
    let y = mapHeight/2.0 - (mapWidth*mercN/(2*M_PI))

    return CGPoint(x: x, y: y)
  }
}

// MARK: UICollectionViewDataSource
extension ViewController: UICollectionViewDataSource {
  func numberOfSections(in collectionView: UICollectionView) -> Int {
    return 1
  }

  func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return astronauts.count
  }
  
  func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "Cell", for: indexPath) as! PersonCell
    cell.nameLabel.text = astronauts[(indexPath as NSIndexPath).item]
    return cell
  }
}

// MARK: UITableViewDataSource
extension ViewController: UITableViewDataSource {
  func numberOfSections(in tableView: UITableView) -> Int {
    return flyByTimes.count
  }

  func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return flyByTimes[section].count
  }

  func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
    let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath) as! FlyByCell
    let flyBy = flyByTimes[(indexPath as NSIndexPath).section][(indexPath as NSIndexPath).row]
    cell.startLabel.text = flyBy.formattedTime
    cell.durationLabel.text = String(format: "%d:%02d", flyBy.duration/60, flyBy.duration%60)
    return cell
  }
}

// MARK: UITableViewDelegate
extension ViewController: UITableViewDelegate {
  func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
    let header = UIView(frame: CGRect(x: 0, y: 0, width: tableView.frame.width, height: 40))
    if let flyBy = flyByTimes[section].first {
      let textLabel = UILabel(frame: header.bounds)
      textLabel.textAlignment = .center
      textLabel.textColor = UIColor(white: 1.0, alpha: 0.5)
      textLabel.font = UIFont.boldSystemFont(ofSize: 32)
      textLabel.text = flyBy.formattedDate.uppercased()
      header.addSubview(textLabel)
    }
    return header
  }
}
