/*
* Copyright (c) 2016 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import Foundation
import CoreLocation

struct Location {
  let lat: Double
  let lng: Double
}

struct FlyBy {
  static var dateFormatter: DateFormatter = {
    let formatter = DateFormatter()
    formatter.dateFormat = "EEE, MMM. dd, yyyy"
    return formatter
  }()

  static var timeFormatter: DateFormatter = {
    let formatter = DateFormatter()
    formatter.dateFormat = "hh:mm:ss a, z"
    return formatter
  }()

  let riseDate: Date
  let duration: Int

  var formattedDate: String {
    return FlyBy.dateFormatter.string(from: riseDate)
  }

  var formattedTime: String {
    return FlyBy.timeFormatter.string(from: riseDate)
  }

  init(riseTime: Int, duration: Int) {
    self.riseDate = Date(timeIntervalSince1970: TimeInterval(riseTime))
    self.duration = duration
  }
}

struct SpaceService {
  static func mainThread(_ completion: @escaping () -> ()) {
    DispatchQueue.main.async {
      completion()
    }
  }

  static func getCurrentISSLocation(_ completion: @escaping ((Location?) -> Void)) {
    let url = URL(string: "http://api.open-notify.org/iss-now.json")!

    URLSession.shared.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data, error == nil else {
        mainThread({ completion(nil) })
        return
      }

      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject

      guard let position = json?["iss_position"] as? [String: Double],
        let lat = position["latitude"],
        let lng = position["longitude"] else {
          mainThread({ completion(nil) })
          return
      }

      let location = Location(lat: lat, lng: lng)
      mainThread({ completion(location) })
      }.resume()
  }

  static func getAstronautsInISS(_ completion: @escaping (([String]) -> Void)) {
    let url = URL(string: "http://api.open-notify.org/astros.json")!

    URLSession.shared.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data, error == nil else {
        mainThread({ completion([]) })
        return
      }

      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject

      guard let peopleJSON = json?["people"] as? [[String: String]] else {
        mainThread({ completion([]) })
        return
      }

      var people = [String]()
      for person in peopleJSON {
        if let craft = person["craft"], craft == "ISS", let name = person["name"] {
          people.append(name)
        }
      }

      mainThread({ completion(people) })
      }.resume()
  }
}

// MARK: Passover Methods
extension SpaceService {
  static func getLatLongForCity(_ city: String, country: String, completion: @escaping (_ error: Error?, _ lat: Double, _ lng: Double) -> Void) {
    let dictionary = ["city": city, "country": country]
    CLGeocoder().geocodeAddressDictionary(dictionary) { (placemarks: [CLPlacemark]?, error: Error?) in
      if let _ = error {
        completion(error, 0, 0)
      } else {
        if let placemarks = placemarks, placemarks.count > 0 {
          let placemark = placemarks.first!
          let lat = placemark.location!.coordinate.latitude
          let lng = placemark.location!.coordinate.longitude
          completion(nil, lat, lng)
        }
      }
    }
  }

  static func getGroupedFlyByTimes(_ lat: Double, lng: Double, completion: @escaping (([[FlyBy]]) -> Void)) {
    let url = URL(string: "http://api.open-notify.org/iss-pass.json?lat=\(lat)&lon=\(lng)&n=25")!

    URLSession.shared.dataTask(with: url) { (data: Data?, response: URLResponse?, error: Error?) -> Void in
      guard let data = data, error == nil else {
        mainThread({ completion([]) })
        return
      }

      let json = try? JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions.allowFragments) as AnyObject

      guard let response = json?["response"] as? [[String: Int]] else {
        mainThread({ completion([]) })
        return
      }

      var flyByData = [[FlyBy]]()
      var currentDayString = ""
      var currentArray = [FlyBy]()

      for time in response {
        if let duration = time["duration"], let riseTime = time["risetime"] {
          let flyBy = FlyBy(riseTime: riseTime, duration: duration)
          let day = flyBy.formattedDate
          if day != currentDayString {
            if currentArray.count > 0 {
              flyByData.append(currentArray)
            }
            currentArray = [FlyBy]()
          }
          currentArray.append(flyBy)
          currentDayString = day
        }
      }

      flyByData.append(currentArray)

      mainThread({completion(flyByData) })
      }.resume()
  }
}
