/*
* Copyright (c) 2015 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import UIKit

class SortingViewController: UIViewController {
  @IBOutlet weak var backgroundImage: UIImageView!
  @IBOutlet weak var sortingImage: UIButton!
  @IBOutlet weak var nextImage: UIButton!

  @IBOutlet weak var leftShoebox: UIView!
  @IBOutlet weak var topShoebox: UIView!
  @IBOutlet weak var bottomShoebox: UIView!
  @IBOutlet weak var rightShoebox: UIView!

  @IBOutlet var movementConstraints: [NSLayoutConstraint]!
  @IBOutlet var imageCenterXConstraint: NSLayoutConstraint!
  @IBOutlet var imageCenterYConstraint: NSLayoutConstraint!
  @IBOutlet var tapGestureRecognizers: [UITapGestureRecognizer]!

  @IBOutlet weak var panGestureRecognizer: UIPanGestureRecognizer!

  var useTapGestures = true
  var imagesToSort = [String]()
  var currentImageIndex = 0
  var readyForNextImage = false
  var centerRegion: CGRect!

  var animator: UIDynamicAnimator!
  var fieldBehavior: SpringFieldsBehavior!

  fileprivate enum Direction: Int {
    case up = 0
    case down
    case left
    case right

    init?(pressType: UIPressType) {
      switch pressType {
      case .upArrow:
        self = .up
      case .downArrow:
        self = .down
      case .leftArrow:
        self = .left
      case .rightArrow:
        self = .right
      default:
        return nil
      }
    }

    init?(swipeDirection: UISwipeGestureRecognizerDirection) {
      if swipeDirection.contains(.up) {
        self = .up
      } else if swipeDirection.contains(.down) {
        self = .down
      } else if swipeDirection.contains(.left) {
        self = .left
      } else if swipeDirection.contains(.right) {
        self = .right
      } else {
        return nil
      }
    }
  }

  // MARK: - Setup

  override func viewDidLoad() {
    super.viewDidLoad()

    resetMovementConstraints()
    imagesToSort = ["swirlysquirrels",
      "chipmunk",
      "cloudzebra",
      "dolphin",
      "koala",
      "muledeer",
      "ornaments",
      "puffin",
      "bearswim"]

    setImages(forIndex: currentImageIndex)
    if useTapGestures {
      setTapGesturesEnabled(true)
      setupSwipeGestures()
    } else {
      setTapGesturesEnabled(false)
//      setupSwipeGestures()

      // The current beta will crash if you try to add a behavior when sortingImage has a zero size, so we force a layout before adding the behavior
      view.layoutIfNeeded()
      animator = UIDynamicAnimator(referenceView: view)
      animator.delegate = self
      fieldBehavior = SpringFieldsBehavior(item: sortingImage, fields: calculateFields())
      animator.addBehavior(fieldBehavior)
//      animator.debugEnabled = true
    }
  }

  fileprivate func setTapGesturesEnabled(_ enabled: Bool) {
    for gesture in tapGestureRecognizers {
      gesture.isEnabled = enabled
    }
    panGestureRecognizer.isEnabled = !enabled
  }

  fileprivate func setupSwipeGestures() {
    // Manually add the swipe gesture recognizers, just as an example.  These could just as easily be added in Interface Builder
    let swipeLeft = UISwipeGestureRecognizer(target: self, action: #selector(handleSwipe(_:)))
    swipeLeft.direction = .left
    sortingImage.addGestureRecognizer(swipeLeft)
    let swipeRight = UISwipeGestureRecognizer(target: self, action: #selector(handleSwipe(_:)))
    swipeRight.direction = .right
    sortingImage.addGestureRecognizer(swipeRight)
    panGestureRecognizer.require(toFail: swipeLeft)
    panGestureRecognizer.require(toFail: swipeRight)
  }

  fileprivate func calculateFields() -> [FieldInfo] {
    let viewHeight = view.bounds.height
    let viewWidth = view.bounds.width
    let sideColumnWidth = leftShoebox.frame.maxX + leftShoebox.frame.minX
    let middleColumnWidth = viewWidth - ((leftShoebox.center.x + (sideColumnWidth / 2)) * 2)
    let middleColumnHeight = (sortingImage.frame.minY - topShoebox.center.y) * 2
    let centerRegionHeight = viewHeight - ((topShoebox.center.y + (middleColumnHeight / 2)) * 2)
    let centerRegionSize = CGSize(width: middleColumnWidth, height: centerRegionHeight)
    centerRegion = view.center.centerOfRect(withSize: centerRegionSize)
    let fields = [FieldInfo(position: leftShoebox.center, size: CGSize(width: sideColumnWidth, height: viewHeight)),
      FieldInfo(position: topShoebox.center, size: CGSize(width: middleColumnWidth, height: middleColumnHeight)),
      FieldInfo(position: view.center, size: centerRegionSize),
      FieldInfo(position: bottomShoebox.center, size: CGSize(width: middleColumnWidth, height: middleColumnHeight)),
      FieldInfo(position: rightShoebox.center, size: CGSize(width: sideColumnWidth, height: viewHeight))]
    return fields
  }

  // MARK: - Private helper methods

  fileprivate func setImages(forIndex index: Int) {
    guard imagesToSort.count > index, let firstImage = UIImage(named: imagesToSort[index]) else { return }
    sortingImage.setBackgroundImage(firstImage, for: UIControlState())
    backgroundImage.image = firstImage

    if imagesToSort.count > index + 1, let secondImage = UIImage(named: imagesToSort[index+1]) {
      nextImage.setBackgroundImage(secondImage, for: UIControlState())
    } else {
      nextImage.setBackgroundImage(UIImage(named: "done"), for: UIControlState())
    }
  }

  fileprivate func moveToNextImage() {
    let stackImage = UIImageView(frame: sortingImage.frame)
    stackImage.image = sortingImage.backgroundImage(for: UIControlState())
    view.insertSubview(stackImage, belowSubview: sortingImage)

    if currentImageIndex == imagesToSort.count-1 {
      sortingImage.isHidden = true
    } else {
      resetMovementConstraints()
      currentImageIndex += 1
      setImages(forIndex: currentImageIndex)
    }
  }

  fileprivate func resetMovementConstraints() {
    for constraint in movementConstraints {
      constraint.isActive = false
    }
    imageCenterXConstraint.isActive = true
    imageCenterYConstraint.isActive = true
  }

  fileprivate func movePhoto(_ direction: Direction) {
    resetMovementConstraints()
    switch direction {
    case .up, .down:
      imageCenterYConstraint.isActive = false
    case .left, .right:
      imageCenterXConstraint.isActive = false
    }

    movementConstraints[direction.rawValue].isActive = true
    UIView.animate(withDuration: 0.5, delay: 0, options: [.curveEaseIn, .curveEaseOut],
      animations: {
        self.view.layoutIfNeeded()
      },
      completion: { finished in
        self.moveToNextImage()
      }
    )
  }

  fileprivate func shufflePhoto(_ direction: Direction) {
    resetMovementConstraints()
    imageCenterXConstraint.isActive = false
    let tempConstraint: NSLayoutConstraint
    if direction == .left {
      tempConstraint = sortingImage.rightAnchor.constraint(equalTo: nextImage.leftAnchor, constant: -20)
    } else {
      tempConstraint = sortingImage.leftAnchor.constraint(equalTo: nextImage.rightAnchor, constant: 20)
    }
    tempConstraint.isActive = true
    UIView.animate(withDuration: 0.25, delay: 0, options: [.curveEaseIn, .curveEaseOut],
      animations: {
        self.view.layoutIfNeeded()
      },
      completion: { finished in
        tempConstraint.isActive = false
        self.imageCenterXConstraint.isActive = true
        UIView.animate(withDuration: 0.25, delay: 0, options: [.curveEaseIn, .curveEaseOut],
          animations: {
            if self.currentImageIndex < self.imagesToSort.count - 1 {
              self.sortingImage.transform = CGAffineTransform(scaleX: 0.8, y: 0.8)
              self.view.bringSubview(toFront: self.nextImage)
            }
            self.view.layoutIfNeeded()
          },
          completion: { finished in
            self.sortingImage.transform = CGAffineTransform.identity
            self.view.bringSubview(toFront: self.sortingImage)
            let elementToMove = self.imagesToSort.remove(at: self.currentImageIndex)
            self.imagesToSort.append(elementToMove)
            self.setImages(forIndex: self.currentImageIndex)
          }
        )
      }
    )
  }

  // MARK: - Gesture actions

  @IBAction func handleTap(_ sender: UITapGestureRecognizer) {
    if sender.state == .recognized {
      let pressType = UIPressType(rawValue: sender.allowedPressTypes.first!.intValue)!
      movePhoto(Direction(pressType: pressType)!)
    }
  }

  func handleSwipe(_ sender: UISwipeGestureRecognizer) {
    if sender.state == .recognized {
      shufflePhoto(Direction(swipeDirection: sender.direction)!)
    }
  }

  @IBAction func handlePan(_ gesture: UIPanGestureRecognizer) {
    switch gesture.state {
    case .began:
      // Disable the behavior while the item is manipulated by the pan recognizer.
      fieldBehavior.isEnabled = false

    case .changed:
      // We don't want the image to get lost off the edge of the screen, so we contain it to the view bounds
      let translation = gesture.translation(in: view)
      let location = sortingImage.center + translation
      sortingImage.center = location.bounded(byRect: view.bounds.insetBy(dx: sortingImage.bounds.width/2, dy: sortingImage.bounds.height/2))

      // This resets the translation so the next time the gesture changes, it will be a relative translation and not cumulative
      gesture.setTranslation(CGPoint.zero, in: view)

    case .cancelled, .ended:
      let velocity = gesture.velocity(in: view)
      fieldBehavior.isEnabled = true
      fieldBehavior.addLinearVelocity(velocity)
      readyForNextImage = true

    default:
      break
    }
  }
}

extension SortingViewController: UIDynamicAnimatorDelegate {
  func dynamicAnimatorDidPause(_ animator: UIDynamicAnimator) {
    if readyForNextImage {
      readyForNextImage = false
      if centerRegion.contains(sortingImage.center) == false {
        moveToNextImage()
        // There's a bug in the current beta that keeps the sortingImage from repositioning after resetting the constraints, so the workaround is to manually set it back here
        sortingImage.center = view.center
      }
    }
  }
}
