/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit

struct VideoListViewIdentifiers {
  static let playAllCell = "playAllCell"
  static let videoCell = "videoCell"
  static let videoHeaderView = "videoHeaderView"
}

class VideoListCollectionViewDataSource: NSObject {
  
  let videoCategories = VideoPlistParser.parsePlist(name: "VideosList")
  
  func videoFor(indexPath: IndexPath) -> Video? {
    
    guard let videoCategory = videoCategoryFor(indexPath: indexPath)
      , indexPath.row < videoCategory.videos.count else { return nil }
    
    return videoCategory.videos[indexPath.row]
  }
  
  func videoCategoryFor(indexPath: IndexPath) -> VideoCategory? {
    
    guard indexPath.section < videoCategories.count else { return nil }
    return videoCategories[indexPath.section]
  }
}

extension VideoListCollectionViewDataSource: UICollectionViewDataSource {
  
  func numberOfSections(in collectionView: UICollectionView) -> Int {
    return videoCategories.count
  }
  
  func collectionView(_ collectionView: UICollectionView,
                      numberOfItemsInSection section: Int) -> Int {
    
    let videoCategory = videoCategories[section]
    return videoCategory.videos.count + 1
  }
  
  func collectionView(_ collectionView: UICollectionView,
                      viewForSupplementaryElementOfKind kind: String,
                      at indexPath: IndexPath) -> UICollectionReusableView {
    
    guard kind == UICollectionElementKindSectionHeader else {
      assertionFailure("Unexpected element kind")
      return UICollectionReusableView()
    }
    return self.collectionView(collectionView, headerViewAtIndexPath: indexPath)
  }
  
  private func collectionView(_ collectionView: UICollectionView,
                              headerViewAtIndexPath indexPath: IndexPath) -> UICollectionReusableView {
    
    let videoCategory = videoCategories[indexPath.section]
    
    let headerView = collectionView.dequeueReusableSupplementaryView(
      ofKind: UICollectionElementKindSectionHeader,
      withReuseIdentifier: VideoListViewIdentifiers.videoHeaderView,
      for: indexPath) as! VideoHeaderView
    
    headerView.label.text = videoCategory.name
    
    return headerView
  }
  
  func collectionView(_ collectionView: UICollectionView,
                      cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    
    if let cell = self.collectionView(collectionView, videoCellForItemAtIndexPath: indexPath) {
      return cell
    }
    
    return self.collectionView(collectionView, playAllCellForItemAtIndexPath: indexPath)
  }
  
  func collectionView(_ collectionView: UICollectionView,
                      videoCellForItemAtIndexPath indexPath:IndexPath) -> VideoCell? {
    
    guard let video = videoFor(indexPath: indexPath) else {
      return nil
    }
    
    let cell = collectionView.dequeueReusableCell(
      withReuseIdentifier: VideoListViewIdentifiers.videoCell,
      for: indexPath) as! VideoCell
    
    cell.video = video
    
    return cell
  }
  
  func collectionView(_ collectionView: UICollectionView,
                      playAllCellForItemAtIndexPath indexPath:IndexPath) -> UICollectionViewCell {
    
    return collectionView.dequeueReusableCell(
      withReuseIdentifier: VideoListViewIdentifiers.playAllCell,
      for: indexPath)
  }
}
