/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import AVFoundation

class QueuePlayerViewController: UIViewController {
  
  private var _playerLayer: AVPlayerLayer?
  private var _queuePlayer: AVQueuePlayer?
  
  var player: AVQueuePlayer? {
    return _queuePlayer
  }
  
  var videoCategory: VideoCategory? {
    didSet {
      guard let videoCategory = videoCategory else {
        return
      }
      setupQueuePlayerWith(videos: videoCategory.videos)
    }
  }    
  
  private func setupQueuePlayerWith(videos: [Video]) {
    let playerItems = videos.map { AVPlayerItem(url: $0.videoURL) }
    _queuePlayer = AVQueuePlayer(items: playerItems)
    _queuePlayer!.play()
    setupPlayerLayer()
  }
  
  private func setupPlayerLayer() {
    
    guard let queuePlayer = _queuePlayer, isViewLoaded else {
      return
    }
    
    _playerLayer?.removeFromSuperlayer()
    
    _playerLayer = AVPlayerLayer(player: queuePlayer)
    _playerLayer!.frame = view.layer.bounds
    view.layer.addSublayer(_playerLayer!)
  }
  
  override func viewDidAppear(_ animated: Bool) {
    super.viewDidAppear(animated)
    setupPlayerLayer()
  }
  
  lazy var playPauseGesture: UITapGestureRecognizer = {
    let playPauseGesture = UITapGestureRecognizer(target: self,
                                                  action: #selector(playPausePressed(_:)))
    let playPauseType = NSNumber(value: UIPressType.playPause.rawValue)
    playPauseGesture.allowedPressTypes = [playPauseType];
    return playPauseGesture
  }()
  
  @objc private func playPausePressed(_ gesture: UITapGestureRecognizer) {
    guard let queuePlayer = _queuePlayer else {
      return
    }
    if queuePlayer.rate == 0 {
      queuePlayer.play()
    } else {
      queuePlayer.pause()
    }
  }
  
  override func willMove(toParentViewController parent: UIViewController?) {
    parent?.view?
      .removeGestureRecognizer(playPauseGesture)
    parent?.view?.addGestureRecognizer(playPauseGesture)
    super.willMove(toParentViewController: parent)
  }
  
  func playVideosFrom(startIndex: Int) {
    guard let videoCategory = videoCategory else {
      return
    }
    let endIndex = videoCategory.videos.count - 1
    let videos = Array(videoCategory.videos[startIndex...endIndex])
    setupQueuePlayerWith(videos: videos)
  }
}

extension PlayAllVideoCategoryViewController: UICollectionViewDelegate {
  func collectionView(_ collectionView: UICollectionView,
                      didSelectItemAt indexPath: IndexPath) {
    let startVideoIndex = indexPath.row
    queuePlayerViewController.playVideosFrom(startIndex: startVideoIndex)
  }
}
