/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit

class VideoListViewController: UIViewController {

  @IBOutlet weak var collectionView: UICollectionView!
  @IBOutlet var collectionViewDataSource: VideoListCollectionViewDataSource!

  @IBOutlet weak var progressView: UIProgressView!

  var currentVideoResourceRequest: NSBundleResourceRequest?
  var selectedIndexPath: IndexPath?
  var currentCategoryResourceRequest: NSBundleResourceRequest?
}

extension VideoListViewController: UICollectionViewDelegate {

  func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    didSelectVideoAt(indexPath)
  }

  func didSelectVideoAt(_ indexPath: IndexPath) {
    selectedIndexPath = indexPath
    progressView.isHidden = false

    currentVideoResourceRequest?.progress.cancel()

    guard var video = collectionViewDataSource.videoFor(indexPath: indexPath),
      let videoCategory = collectionViewDataSource.videoCategoryFor(indexPath: indexPath) else {
        return
    }

    currentVideoResourceRequest = ResourceManager.shared
      .requestVideoWith(tag: video.videoName, progressObserver: self,
        onSuccess: { [weak self] in
          self?.progressView.isHidden = true
          guard let currentVideoResourceRequest =
            self?.currentVideoResourceRequest else { return }
          video.videoURL = currentVideoResourceRequest.bundle
            .url(forResource: video.videoName, withExtension: "mp4")
          let viewController = PlayVideoViewController
            .instanceWith(video: video, videoCategory: videoCategory)
          self?.navigationController?.pushViewController(viewController,
            animated: true)
        },
        onFailure: { [weak self] error in
          self?.progressView.isHidden = true
          self?.handleDownloadingError(error as NSError)
        }
    )

    currentCategoryResourceRequest?.endAccessingResources()
    currentCategoryResourceRequest = ResourceManager
      .shared.requestCategoryWith(tag: videoCategory.name)
  }

  func handleDownloadingError(_ error: NSError) {
    switch error.code{
    case NSBundleOnDemandResourceOutOfSpaceError:
      let message = "You don't have enough storage left to download this resource."
      let alert = UIAlertController(title: "Not Enough Space",
        message: message,
        preferredStyle: .alert)
      alert.addAction(UIAlertAction(title: "OK",
        style: .cancel, handler: nil))
      present(alert, animated: true, completion: nil)
    case NSBundleOnDemandResourceExceededMaximumSizeError:
      assert(false, "The bundle resource was too large.")
    case NSBundleOnDemandResourceInvalidTagError:
      assert(false, "The requested tag(s) (\(currentVideoResourceRequest?.tags ?? [""])) does not exist.")
    default:
      assert(false, error.description)
    }
  }

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)

    currentVideoResourceRequest?.endAccessingResources()
    currentVideoResourceRequest = nil

    if let selectedIndexPath = selectedIndexPath,
      selectedIndexPath.item + 1 == collectionView
        .numberOfItems(inSection: selectedIndexPath.section) {

          currentCategoryResourceRequest?.endAccessingResources()
          currentCategoryResourceRequest = nil
    }
  }
  
  override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {
    if context == progressObservingContext
      && keyPath == "fractionCompleted" {

      OperationQueue.main.addOperation {
        self.progressView.progress
          = Float((object as! Progress).fractionCompleted)
      }
    }
  }

}
