/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Foundation
import CloudKit

let GlobalModel = Model()

let MyMovieNotification = Notification.Name("movies.mymovies")
let MyQueueNotification = Notification.Name("movies.myqueue")

class Model {
  var container: CKContainer { return CKContainer.default() }
  var publicDatabase: CKDatabase { return container.publicCloudDatabase }
  var privateDatabase: CKDatabase { return container.privateCloudDatabase }

  private func fetchUserId(_ completion:
    @escaping (CKRecordID) -> Void) {
    container.fetchUserRecordID() { recordId, error in
      if let id = recordId {
        completion(id)
      } else {
        showError(error as NSError?, inController: nil)
      }
    }
  }

  func withUserRecord(_ completion: @escaping (CKRecord) -> Void) {
    fetchUserId { userId in
      self.fetchUserRecord(userId, completion: completion)
    }
  }

  private func fetchUserRecord(_ userId: CKRecordID,
                    completion: @escaping (CKRecord) -> Void) {
    privateDatabase.fetch(withRecordID: userId) { record, error in
      if let user = record {
        completion(user)
      } else {
        showError(error as NSError?, inController: nil)
      }
    }
  }

}

// MARK: - Adding and Finding a Movie
extension Model {
  func createNewMovie(_ movie: Movie,
                      handler: @escaping (CKRecord) -> Void) {
    let movieRecord = movie.record
    publicDatabase.save(movieRecord) { record, error in
      guard let record = record, error == nil else {
        showError(error as NSError?, inController: nil)
        return
      }
      handler(record)
    }
  }

  func movieForTitle(_ title: String,
                     completion: @escaping (CKRecord?, NSError?) -> ()) {
    // 1
    let titlePredicate = NSPredicate(format: "title == %@", title)
    // 2
    let query = CKQuery(recordType: MovieRecordType,
                        // 3
      predicate: titlePredicate)
    publicDatabase.perform(query, inZoneWith: nil) { records,
      error in
      guard let records = records, records.count > 0 &&
        error == nil else {
          completion(nil, error as? NSError)
          return
      }
      let record = records[0]
      completion(record, nil)
    }
  }

  func withMovieRecord(_ movie: Movie,
           completion: @escaping (CKRecord) -> Void) {
    // 1
    if let movie = movie as? CKRecord {
      completion(movie)
      return
    }
    // 2
    movieForTitle(movie.title) { record, error in
      // 3
      if let record = record {
        completion(record)
        return
      }
      if let error = error {
        // 4
        if error.domain == CKErrorDomain &&
          error.code == CKError.Code.unknownItem.rawValue {
          self.createNewMovie(movie) { record in
            completion(record)
          }
        } else {
          showError(error, inController: nil)
        }
      } else { // 5
        self.createNewMovie(movie) {
          record in completion(record)
        }
      }
    }
  }
}

// MARK: - My Movies
extension Model {

  func fetchMovies(_ completion:
    @escaping ([CKRecord]?, NSError?) -> ()) {
    withUserRecord() { user in
      // 1
      guard let movieReferences = user["movies"] as? [CKReference],
        movieReferences.count > 0  else { return }
      // 2
      let fetch = CKFetchRecordsOperation(
        recordIDs: movieReferences.map { $0.recordID })
      // 3
      fetch.fetchRecordsCompletionBlock = { records, error in
        guard let movieRecords = records else {
          showError(error as NSError?, inController: nil)
          return
        }
        // 4
        let fetchedMovies: [CKRecord] = Array(movieRecords.values)
        let sortedMovies = fetchedMovies.sorted {
          return ($0.title as NSString)
            .caseInsensitiveCompare($1.title) == .orderedAscending }
        // 5
        DispatchQueue.main.async {
          completion(sortedMovies, nil)
        }
      }
      // 6
      self.publicDatabase.add(fetch)
    }
  }

  func addToMyMovies(_ movie: Movie) {
    withMovieRecord(movie) { movieRecord in
      self.addRecordToMyMovies(movieRecord)
    }
  }
  
  private func addRecordToMyMovies(_ movieRecord: CKRecord) {
    // 1
    withUserRecord() { user in
      // 2
      var movies = user["movies"] as? [CKReference]
      if movies == nil {
        movies = [CKReference]()
      }
      // 3
      let movieReference = CKReference(record: movieRecord,
                                       action: .none)
      movies?.append(movieReference)
      // 4
      user["movies"] = movies as NSArray?
      // 5
      self.privateDatabase.save(user) { record, error in
        if error == nil {
          // 6
          DispatchQueue.main.async {
            NotificationCenter.default.post(
              name: MyMovieNotification,
              object: self)
          }
        } else {
          showError(error as NSError?, inController: nil)
        }
      }
    }
  }
  
  func removeFromMyMovies(_ movie: Movie) {
    let error = NSError(domain: "MoviesMoviesMovies", code: 0, userInfo: [NSLocalizedDescriptionKey : "This method has not been implemented yet"])
    showError(error, inController: nil)
  }

}

// MARK: - My Queue
extension Model {
  func fetchQueue(_ completion: (_ movies: [CKRecord]?, _ error: NSError?) -> ()) {
    // fill out with code fetch the my queue list
  }

  func addToMyQueue(_ movie: Movie) {
    let error = NSError(domain: "MoviesMoviesMovies", code: 0, userInfo: [NSLocalizedDescriptionKey : "This method has not been implemented yet"])
    showError(error, inController: nil)
  }

  private func addRecordToMyQueue(_ movieRecord: CKRecord) {
    let error = NSError(domain: "MoviesMoviesMovies", code: 0, userInfo: [NSLocalizedDescriptionKey : "This method has not been implemented yet"])
    showError(error, inController: nil)
  }

  func updateQueue(_ newQueue: [CKRecord]) {
    let error = NSError(domain: "MoviesMoviesMovies", code: 0, userInfo: [NSLocalizedDescriptionKey : "This method has not been implemented yet"])
    showError(error, inController: nil)
  }
}

// MARK: - Recommendations
extension Model {

  func getTopTen(_ completion: ([CKRecord], NSError?) -> Void) {
  }

}

// MARK: - Rating
extension Model {

  func setRating(_ movie: Movie, rating: Int) {
    // Ratings are in the value of 1 to 5 stars. If the rating is zero, then it ratins is unset and it should be deleted
    let error = NSError(domain: "MoviesMoviesMovies", code: 0, userInfo: [NSLocalizedDescriptionKey : "This method has not been implemented yet"])
    showError(error, inController: nil)
  }

}
