/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import CloudKit

class MyQueueViewController: UICollectionViewController {

  override func viewDidLoad() {
    super.viewDidLoad()

    NotificationCenter.default.addObserver(self, selector: #selector(MyQueueViewController.reload), name: MyQueueNotification, object: nil)
    reload()
  }

  var queue: [CKRecord]? {
    didSet {
      collectionView?.reloadData()
    }
  }
  var movingIndexPath: IndexPath?
  var nextIndexPath: IndexPath?
  var shouldMoveCell = false

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    reload()
  }

  func reload() {
    GlobalModel.fetchQueue() { movieRecords, error in
      if let newMovies = movieRecords {
        self.queue = newMovies
      } else {
        showError(error, inController: self)
      }
    }
  }
  
  internal func stopShake() {
    guard let moving = movingIndexPath else { return }
    GlobalModel.updateQueue(queue!)
    let cell = collectionView?.cellForItem(at: moving)
    cell?.layer.removeAnimation(forKey: "wiggle")
    movingIndexPath = nil
  }
  
  override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    guard !isEditing else {
      isEditing = false
      collectionView.endInteractiveMovement()
      stopShake()
      return
    }

    let movie = queue![(indexPath as NSIndexPath).row]

    let actionController = movie.movieActionController(presentingController: self, actionsToLeaveOut: .addToQueue)
    present(actionController, animated: true, completion: nil)
  }
}

extension MyQueueViewController {
  override func numberOfSections(in collectionView: UICollectionView) -> Int {
    return 1
  }

  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return queue?.count ?? 0
  }

  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: MovieCell.Identifier, for: indexPath) as! MovieCell

    let movie = queue![(indexPath as NSIndexPath).row]
    cell.label.text = movie.title
    if let posterURL = movie.poster, let image = UIImage(contentsOfFile: posterURL) {
      cell.imageView.image = image
    } else {
      cell.imageView.image = UIImage(named: "posterPlaceholder")
    }

    let longPress = UILongPressGestureRecognizer(target: self, action: #selector(MyQueueViewController.longpress(_:)))
    cell.addGestureRecognizer(longPress)
    return cell
  }

  func longpress(_ press: UILongPressGestureRecognizer) {
    press.view?.layer.add(makeShakeAnimation(), forKey: "wiggle")
    isEditing = true
    movingIndexPath = collectionView?.indexPath(for: press.view as! UICollectionViewCell)
    shouldMoveCell = true
  }

  override func collectionView(_ collectionView: UICollectionView, shouldUpdateFocusIn context: UICollectionViewFocusUpdateContext) -> Bool {
    guard let moving = movingIndexPath else { return true }
    guard shouldMoveCell  else { return !isEditing }

    switch context.focusHeading {
    case UIFocusHeading.right:
      nextIndexPath = IndexPath(row: (moving as NSIndexPath).row + 1, section: (moving as NSIndexPath).section)
      updateRow(moving)
    case UIFocusHeading.left:
      nextIndexPath = IndexPath(row: (moving as NSIndexPath).row - 1, section: (moving as NSIndexPath).section)
      updateRow(moving)
    default:
      isEditing = false
      stopShake()
    }
    return !isEditing
  }

  private func updateRow(_ moving: IndexPath) {
    collectionView!.moveItem(at: moving, to: nextIndexPath!)
    let movieToMove = queue?.remove(at: movingIndexPath!.row)
    queue!.insert(movieToMove!, at: nextIndexPath!.row )
    let disp = queue!.map { $0.title }
    print(disp)
    shouldMoveCell = false
    let delayTime = DispatchTime.now() + Double(Int64(0.3 * Double(NSEC_PER_SEC))) / Double(NSEC_PER_SEC)
    DispatchQueue.main.asyncAfter(deadline: delayTime) {
      self.shouldMoveCell = true
      self.movingIndexPath = self.nextIndexPath
    }
  }
}

/** Helper function to make a shake animation */
private func makeShakeAnimation() -> CAAnimation {
  let animation = CAKeyframeAnimation(keyPath: "transform.rotation.z")
  let angle: CGFloat = 0.06
  animation.values = [-angle, angle]

  animation.autoreverses = true
  animation.duration = 0.1
  animation.repeatCount = Float.infinity
  return animation
}
