/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import CloudKit

class MovieViewController: UIViewController {

  @IBOutlet weak var poster: UIImageView!

  @IBOutlet weak var movieTitle: UILabel!
  @IBOutlet weak var info: UILabel!
  @IBOutlet weak var synopsis: UILabel!

  @IBOutlet weak var addToQueueButton: UIButton!
  @IBOutlet weak var addToMoviesButton: UIButton!
  @IBOutlet weak var rateMovieButton: UIButton!

  @IBOutlet weak var addToQueueLabel: UILabel!
  @IBOutlet weak var addToMoviesLabel: UILabel!

  var movie: Movie!
  var removeFromMyMovies = false

  override func viewDidLoad() {
    super.viewDidLoad()

    // Do any additional setup after loading the view.
    movieTitle.text = movie.title
    info.text = ([movie.genre, movie.year] as [Any?])
      .flatMap { $0 }
      .map { String(describing: $0) }
      .joined(separator: " \t ")
    synopsis.text = movie.synopsis

    if let posterURL = movie.poster,
      let url = URL(string: posterURL),
      let data = (try? Data(contentsOf: url)) ?? (try? Data(contentsOf: URL(fileURLWithPath: posterURL))),
      let image = UIImage(data: data) {
        poster.image = image
    } else {
      poster.image = UIImage(named: "posterPlaceholder")
    }

    let (fg, bg) = colorAtCorner(poster.image!)
    view.backgroundColor = bg
    movieTitle.textColor = fg
    info.textColor = fg
    synopsis.textColor = fg

    guard movie is CKRecord else { return }
    // if this is showing a record, let the user remove rather than add it from the queue, if that is appropriate

    GlobalModel.withUserRecord() { userRecord in
      // if this already in 'My Movies' the action should remove it instead
      if let movies = userRecord["movies"] as? [CKReference], (movies.filter { $0.recordID == (self.movie as! CKRecord).recordID }.count > 0) {

        DispatchQueue.main.async {
          self.removeFromMyMovies = true
          self.addToMoviesLabel.text = "Remove from My Movies"
          self.addToMoviesButton.backgroundColor = UIColor.moviesRedColor
        }
      }
    }
  }

  @IBAction func rateMovie(_ sender: AnyObject) {
    let ratingVC = RatingViewController.makeRatingController(movie)
    present(ratingVC, animated: true, completion: nil)
  }

  @IBAction func addToMovies(_ sender: AnyObject) {
    if removeFromMyMovies {
      GlobalModel.removeFromMyMovies(movie)
    } else {
      GlobalModel.addToMyMovies(movie)
    }
    jumpToMyMovies()

  }

  @IBAction func addToQueue(_ sender: AnyObject) {
    GlobalModel.addToMyQueue(movie)
    jumpToMyMovies()
  }
}

func colorAtCorner(_ image: UIImage) -> (text: UIColor, background: UIColor) {
  let imageRef:CGImage = image.cgImage!

  let colorSpace = CGColorSpaceCreateDeviceRGB()
  let bytesPerPixel = 4
  let bytesPerRow = 1 * bytesPerPixel
  let bitsPerComponent = 8
  var pixelData: [UInt8] = [0, 0, 0, 0]
  let pixelPointer = UnsafeMutableRawPointer(mutating: pixelData)

  let context = CGContext(data: pixelPointer, width: 1, height: 1, bitsPerComponent: bitsPerComponent, bytesPerRow: bytesPerRow, space: colorSpace, bitmapInfo: CGImageAlphaInfo.premultipliedLast.rawValue | CGBitmapInfo.byteOrder32Big.rawValue)

  context?.setBlendMode(CGBlendMode.copy)
  context?.draw(imageRef, in: CGRect(x: 0, y: 0, width: 1, height: 1), byTiling: false)

  let red = CGFloat(pixelData[0]) / 255
  let green = CGFloat(pixelData[1]) / 255
  let blue = CGFloat(pixelData[2]) / 255

  let darkness = (Int(pixelData[0]) * 299 + Int(pixelData[1]) * 587 + Int(pixelData[2]) * 114) / 1000
  let textColor = darkness < 125 ? UIColor.white : UIColor.black

  return (textColor, UIColor(red: red, green: green, blue: blue, alpha: 1))
}
