/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import CloudKit

class MyMoviesViewController: UICollectionViewController {

  @IBOutlet var messageView: UIView!
  @IBOutlet weak var messagesSubtitle: UILabel!
  @IBOutlet weak var messagesTitle: UILabel!


  override func viewDidLoad() {
    super.viewDidLoad()

    NotificationCenter.default.addObserver(self, selector: #selector(MyMoviesViewController.reload), name: MyMovieNotification, object: nil)
    reload()
  }

  var movies: [CKRecord]? {
    didSet {
      collectionView?.reloadData()
      updateMessage()

    }
  }

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    updateMessage()
  }

  private func addMessage(_ title: String, subtitle: String) {
    messagesTitle.text = title
    messagesSubtitle.text = subtitle

    messageView.frame = view.bounds
    view.addSubview(messageView)
  }

  private func removeMessage() {
    messageView.removeFromSuperview()
  }

  private func showNoMovieMessage() {
    if movies?.count == 0 {
      addMessage("You do not have any movies yet.", subtitle: "Search to add movies.")
    } else {
      if messageView.superview != nil {
        removeMessage()
        reload()
      }
    }
  }

  private func showNoAccountMessage(_ status: CKAccountStatus) {
    switch status {
    case .available:
      removeMessage()
    case .couldNotDetermine, .noAccount:
      addMessage("You are not signed in to iCloud.", subtitle: "Sign in to iCloud in the Settings app.")
    case .restricted:
      addMessage("You cannot sign into iCloud.", subtitle: "Movies, Movies, Movies requires iCloud access and you are not allowed to sign in.")
    }
  }

  private func updateMessage() {
    GlobalModel.checkAccountStatus() { status in
      if status == .available {
        self.showNoMovieMessage()
      } else {
        self.showNoAccountMessage(status)
      }
    }
  }

  func reload() {
    GlobalModel.fetchMovies() { movieRecords, error in
      if let newMovies = movieRecords {
        self.movies = newMovies
      } else {
        showError(error, inController: self)
      }
    }
  }
}

extension MyMoviesViewController {
  override func numberOfSections(in collectionView: UICollectionView) -> Int {
    return 1
  }

  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return movies?.count ?? 0
  }

  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: MovieCell.Identifier, for: indexPath) as! MovieCell

    let movie = movies![(indexPath as NSIndexPath).row]
    cell.label.text = movie.title
    if let posterURL = movie.poster, let image = UIImage(contentsOfFile: posterURL) {
      cell.imageView.image = image
    } else {
      cell.imageView.image = UIImage(named: "posterPlaceholder")
    }
    return cell
  }

  override func collectionView(_ collectionView: UICollectionView,
                               didSelectItemAt indexPath: IndexPath) {
    let movie = movies![indexPath.row]
    GlobalModel.withMovieRecord(movie) { fullMovieRecord in
      DispatchQueue.main.async {
        let actionController = fullMovieRecord
          .movieActionController(presentingController: self,
                                 actionsToLeaveOut: .addToMovies)
        self.present(actionController,
                     animated: true,
                     completion: nil)
      }
    }
  }
}
