/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Foundation

struct OMDBIMovie: Movie {

  let title: String
  let poster: String?
  let synopsis: String?
  let genre: String?
  let year: Int?

  init?(json: [String: AnyObject]) {
    guard let title = json["Title"] as? String else { return nil }
    self.title = title
    poster = json["Poster"] as? String
    synopsis = json["Plot"] as? String

    // genres in OMDB are a ,-separated list, but this app only has one genre per movie
    if let genres =  json["Genre"] as? String {
      genre = genres.components(separatedBy: ",").first
    } else {
      genre = nil
    }
    if let yearStr =  json["Year"] as? String {
      year = Int(yearStr)
    } else {
      year = nil
    }
  }
}

class OMDBIApi: MovieSearchAPI {
  var title: String { return "OMDB" }

  var task: URLSessionDataTask?

  func searchByTitle(_ title: String, callback: @escaping (_ movie: Movie) -> ()) {
    task?.cancel()

    let escapedTitle = (title + "*").addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
    let urlStr = "http://www.omdbapi.com/?t=" + escapedTitle + "&y=&plot=short&r=json"
    let configuration = URLSessionConfiguration.default
    let session = URLSession(configuration: configuration)
    let request = URLRequest(url: URL(string: urlStr)!)
    task = session.dataTask(with: request) {data, response, error in
      guard error == nil && data != nil else { return }

      let dataStr = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)
      print(dataStr)

      if let json = try? JSONSerialization.jsonObject(with: data!, options: JSONSerialization.ReadingOptions(rawValue: 0)) {
        if let movie = OMDBIMovie(json: json as! [String: AnyObject]) {
          DispatchQueue.main.async {
            callback(movie)
          }
        }
      }
    }
    task?.resume()
  }
}
