/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Foundation
import UIKit

class SearchResultsController: UICollectionViewController {

  // MARK: UISearchResultsUpdating

  let apis: [MovieSearchAPI] = [OMDBIApi(), CloudKitAPI()]
  var items = [[Movie](), [Movie]()]

  override func numberOfSections(in collectionView: UICollectionView) -> Int {
    return apis.count
  }

  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return items[section].count
  }


  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    // Dequeue a cell from the collection view.
    return collectionView.dequeueReusableCell(withReuseIdentifier: MovieCell.Identifier, for: indexPath)
  }

  // MARK: UICollectionViewDelegate

  override func collectionView(_ collectionView: UICollectionView, willDisplay cell: UICollectionViewCell, forItemAt indexPath: IndexPath) {
    guard let cell = cell as? MovieCell else { return }

    let movie = items[(indexPath as NSIndexPath).section][(indexPath as NSIndexPath).row]
    cell.label.text = movie.title
    cell.imageView.alpha = 1.0


    if let posterURL = movie.poster,
      let url = URL(string: posterURL),
      let data = (try? Data(contentsOf: url)) ?? (try? Data(contentsOf: URL(fileURLWithPath: posterURL))),
      let image = UIImage(data: data) {
        cell.imageView.image = image
    } else {
      cell.imageView.image = UIImage(named: "posterPlaceholder")
      cell.imageView.layer.shadowColor = UIColor.green.cgColor
    }
  }

  override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    let movie = items[(indexPath as NSIndexPath).section][(indexPath as NSIndexPath).row]

    let actionController = movie.movieActionController(presentingController: self, actionsToLeaveOut: .remove)
    present(actionController, animated: true, completion: nil)
  }

  override func collectionView(_ collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, at indexPath: IndexPath) -> UICollectionReusableView {
    let headerView = collectionView.dequeueReusableSupplementaryView(ofKind: UICollectionElementKindSectionHeader, withReuseIdentifier: "Header", for: indexPath)
    let title = apis[(indexPath as NSIndexPath).section].title
    (headerView.viewWithTag(100) as! UILabel).text = title
    return headerView
  }

}

extension SearchResultsController: UISearchResultsUpdating {
  func updateSearchResults(for searchController: UISearchController) {
    guard let filterString = searchController.searchBar.text else { return }
    guard (filterString as NSString).length > 3 else { return }

    for (index, api) in apis.enumerated() {
      api.searchByTitle(filterString) { movie in
        self.items[index] = [movie]
        self.collectionView?.reloadData()
      }
    }
  }

}
