/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

protocol AssetPickerDelegate {
  func assetPickerDidFinishPickingAssets(_ selectedAssets: [PHAsset])
  func assetPickerDidCancel()
}

class AssetGridViewController: UICollectionViewController {
  
  // Segues
  enum Segues: String {
    case SegueDetailToSlideShows
    case SegueAssetDetail
  }
  
  // MARK: - Variables
  var assetsFetchResults: PHFetchResult<AnyObject>?
  var selectedAssets: SelectedAssets!
  var delegate: AssetPickerDelegate?
  
  fileprivate let reuseIdentifier = "AssetCell"
  private let slideshowsSegue = "showSlideshows"
  
  fileprivate var editMode = false
  
  private var doneButton: UIBarButtonItem!
  private var editModeEditButton: UIBarButtonItem!
  private var editModeDoneButton: UIBarButtonItem!
  
  override func viewDidLoad() {
    super.viewDidLoad()
    self.splitViewController?.preferredDisplayMode = .allVisible
    collectionView!.allowsMultipleSelection = true
    doneButton = UIBarButtonItem(title: "Create Slideshow", style: .done, target: self, action: #selector(donePressed(_:)))
    editModeEditButton = UIBarButtonItem(title: "Select Mode", style: .plain, target: self, action: #selector(editModeToggled(_:)))
    editModeDoneButton = UIBarButtonItem(title: "Edit Mode", style: .done, target: self, action: #selector(editModeToggled(_:)))
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    
    collectionView!.reloadData()
    updateSelectedItems()
    updateEditButton()
  }
  
  override func viewDidAppear(_ animated: Bool) {
    super.viewDidAppear(animated)
  }
  
  override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
    super.viewWillTransition(to: size, with: coordinator)
    collectionView!.reloadData()
    updateSelectedItems()
  }
  
  // MARK: - Navigation
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    if segue.identifier == Segues.SegueDetailToSlideShows.rawValue {
      delegate?.assetPickerDidFinishPickingAssets(selectedAssets.assets)
    } else if segue.identifier == Segues.SegueAssetDetail.rawValue {
      // TBD: Edit asset
    }
  }
  
  override func shouldPerformSegue(withIdentifier identifier: String, sender: Any?) -> Bool {
    // Don't view details assets unless in edit mode
    if identifier == Segues.SegueAssetDetail.rawValue && !editMode {
      return false
    }
    return true
  }
  
  // MARK: - Button handlers
  func donePressed(_ sender: UIBarButtonItem) {
    performSegue(withIdentifier: Segues.SegueDetailToSlideShows.rawValue, sender: sender)
  }
  
  func editModeToggled(_ sender: UIBarButtonItem) {
    editMode = !editMode
    updateEditButton()
  }
  
  // MARK: - Private helper methods
  fileprivate func currentAsset(at index:NSInteger) -> PHAsset {
    if let fetchResult = assetsFetchResults {
      return fetchResult[index] as! PHAsset
    } else {
      return selectedAssets.assets[index]
    }
  }

  private func updateSelectedItems() {
    guard selectedAssets != nil else { return }
    // Select the selected items
    if let fetchResult = assetsFetchResults {
      for asset in selectedAssets.assets {
        let index = fetchResult.index(of: asset)
        if index != NSNotFound {
          let indexPath = IndexPath(item: index, section: 0)
          collectionView!.selectItem(at: indexPath,
                                     animated: false, scrollPosition: UICollectionViewScrollPosition())
        }
      }
    } else {
      for i in 0..<selectedAssets.assets.count {
        let indexPath = IndexPath(item: i, section: 0)
        collectionView!.selectItem(at: indexPath,
                                   animated: false, scrollPosition: UICollectionViewScrollPosition())
      }
    }
    updateDoneButton()
  }
  
  fileprivate func updateDoneButton() {
    guard selectedAssets != nil else { return }
    // Add a done button when there are selected assets
    if selectedAssets.assets.count > 0 {
      self.navigationItem.rightBarButtonItem = doneButton
    } else {
      self.navigationItem.rightBarButtonItem = nil
    }
  }
  
  private func updateEditButton() {
    var assetCount = 0
    if let fetchResult = assetsFetchResults {
      assetCount = fetchResult.count
    } else if selectedAssets != nil {
      assetCount = selectedAssets.assets.count
    }
    if assetCount > 0 {
      self.navigationItem.leftBarButtonItem = editMode ? editModeDoneButton : editModeEditButton
    } else {
      self.navigationItem.leftBarButtonItem = nil
    }
  }
  
}

// MARK: - UICollectionViewDelegate, UICollectionViewDataSource
extension AssetGridViewController {
  override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    if editMode { return }
    // Update selected Assets
    let asset = currentAsset(at: indexPath.item)
    selectedAssets.assets.append(asset)
    let cell = collectionView.cellForItem(at: indexPath) as! GridViewCell
    cell.checkMarkSelected(true)
    updateDoneButton()
  }
  
  override func collectionView(_ collectionView: UICollectionView, didDeselectItemAt indexPath: IndexPath) {
    if editMode { return }
    // Update de-selected Assets
    let assetToDelete = currentAsset(at: indexPath.item)
    selectedAssets.assets = selectedAssets.assets.filter { asset in
      !(asset == assetToDelete)
    }
    if assetsFetchResults == nil {
      collectionView.deleteItems(at: [indexPath])
    }
    let cell = collectionView.cellForItem(at: indexPath) as! GridViewCell
    cell.checkMarkSelected(false)
    updateDoneButton()
  }
  
  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int  {
    if let fetchResult = assetsFetchResults {
      return fetchResult.count
    } else if selectedAssets != nil {
      return selectedAssets.assets.count
    }
    return 0
  }
  
  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: reuseIdentifier, for: indexPath) as! GridViewCell
    
    // TBD: Populate Cell
    
    return cell
  }
}

// MARK: - UICollectionViewDelegateFlowLayout
extension AssetGridViewController: UICollectionViewDelegateFlowLayout {
  func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
    var thumbsPerRow: Int
    switch collectionView.bounds.size.width {
    case 0..<400:
      thumbsPerRow = 4
    case 400..<600:
      thumbsPerRow = 5
    case 600..<800:
      thumbsPerRow = 6
    case 800..<1200:
      thumbsPerRow = 7
    default:
      thumbsPerRow = 4
    }
    let width = collectionView.bounds.size.width / CGFloat(thumbsPerRow)
    return CGSize(width: width,height: width)
  }
}

