/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import MultipeerConnectivity

class ViewController: UIViewController {
  
  @IBOutlet weak var imageView: UIImageView!
  
  fileprivate var slideshow: Slideshow!
  private let SegueToSlideshowEdit = "SegueToSlideshowEdit"
  fileprivate var editSlideshow: Bool = true
  
  override func viewDidLoad() {
    super.viewDidLoad()
    NotificationCenter.default.addObserver(self, selector: #selector(noteSlideshow), name: .dataReceived, object: nil)
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    imageView.image = slideshow?.getImage()
  }
  
  deinit {
    NotificationCenter.default.removeObserver(self)
  }
  
  // MARK: - Navigation
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    if segue.identifier == SegueToSlideshowEdit {
      let destination = segue.destination as! SlideshowEditViewController
      destination.slideshow = slideshow
    }
  }
  
  // MARK: - Actions
  @IBAction func customizeThemePressed(_ sender: AnyObject) {
    if PeerToPeerConnectionManager.sharedInstance.isConnected() {
      // Show the slideshow edit controller
      showEditSlideshow()
    } else {
      // Show the peer to peer browser
      editSlideshow = true
      showBrowser()
    }
  }
  
  @IBAction func addPhotoPressed(_ sender: AnyObject) {
    if PeerToPeerConnectionManager.sharedInstance.isConnected() {
      // Show the image picker controller
      showAddPhoto()
    } else {
      // Show the peer to peer browser
      editSlideshow = false
      showBrowser()
    }
  }
  
  func noteSlideshow() {
    slideshow = PeerToPeerConnectionManager.sharedInstance.slideshow
    imageView.image = slideshow?.getImage()
  }
  
  // MARK: - Private helper methods
  private func showBrowser() {
    let browserViewController = PeerToPeerConnectionManager.sharedInstance.setupPeerToPeerBrowser()
    browserViewController.delegate = self
    self.present(browserViewController, animated: true, completion: nil)
  }
  
  fileprivate func showEditSlideshow() {
    performSegue(withIdentifier: SegueToSlideshowEdit, sender: nil)
  }
  
  fileprivate func showAddPhoto() {
    if isSimulator() {
      showImagePicker(sourceType: .photoLibrary)
    } else {
      // Give the user a choice using an action sheet
      let actionSheet = UIAlertController(
        title: nil,
        message: nil,
        preferredStyle: .actionSheet)
      actionSheet.addAction(UIAlertAction(
        title: "Camera",
        style: .default,
        handler: {_ in
          self.showImagePicker(sourceType: .camera)
      }))
      actionSheet.addAction(UIAlertAction(
        title: "Photo Libray",
        style: .default,
        handler: {_ in
          self.showImagePicker(sourceType: .photoLibrary)
      }))
      actionSheet.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
      self.present(actionSheet, animated: true, completion: nil)
    }
  }
  
  private func showImagePicker(sourceType: UIImagePickerControllerSourceType) {
    // Initialize an image picker controller
    let imagePickerController = UIImagePickerController()
    imagePickerController.sourceType = sourceType
    imagePickerController.isEditing = true
    imagePickerController.delegate = self
    self.present(imagePickerController, animated: true, completion: nil)
  }
  
  private func isSimulator() -> Bool {
    return TARGET_OS_SIMULATOR != 0
  }
  
}

// MARK: - MCBrowserViewControllerDelegate
extension ViewController: MCBrowserViewControllerDelegate {
  func browserViewControllerDidFinish(_ browserViewController: MCBrowserViewController) {
    browserViewController.dismiss(animated: true, completion: nil)
    if slideshow != nil {
      if editSlideshow {
        showEditSlideshow()
      } else {
        showAddPhoto()
      }
    }
  }
  
  func browserViewControllerWasCancelled(_ browserViewController: MCBrowserViewController) {
    browserViewController.dismiss(animated: true, completion: nil)
  }
}

// MARK: - UIImagePickerControllerDelegate
extension ViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
  func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
    let image = info[UIImagePickerControllerOriginalImage] as! UIImage
    // Send photo out
    PeerToPeerConnectionManager.sharedInstance.send(image, to: slideshow)
    // Dismiss picker
    picker.dismiss(animated: true, completion: nil)
  }
}
