/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

class AssetDetailViewController: UIViewController {
  
  @IBOutlet weak var imageView: UIImageView!
  @IBOutlet weak var favoriteButton: UIButton!
  @IBOutlet weak var deleteButton: UIButton!
  
  // MARK: - Variables
  var photoAsset: PHAsset!
  var photoLocalIdentifier: String!
  
  private var targetSize: CGSize {
    let scale = UIScreen.main.scale
    return CGSize(width: imageView.bounds.width * scale,
                  height: imageView.bounds.height * scale)
  }
  fileprivate let symbolFavorite = "\u{2665}"
  fileprivate let symbolUnfavorite = "\u{2661}"
  fileprivate let symbolDelete = "\u{1F5D1}"
  
  override func viewDidLoad() {
    super.viewDidLoad()
    PHPhotoLibrary.shared().register(self)
    displayImage()
  }
  
  deinit {
    PHPhotoLibrary.shared().unregisterChangeObserver(self)
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    guard let asset = photoAsset else { return }
    favoriteButton.setTitle(asset.isFavorite ? symbolFavorite : symbolUnfavorite, for: .normal)
    deleteButton.setTitle(symbolDelete, for: .normal)
    deleteButton.isEnabled = photoAsset.canPerform(.delete)
  }
  
  // MARK: - Actions
  @IBAction func favoritePressed(_ sender: UIButton) {
    PHPhotoLibrary.shared().performChanges({
      let request = PHAssetChangeRequest(for: self.photoAsset)
      request.isFavorite = !self.photoAsset.isFavorite
      }, completionHandler: nil)
  }
  
  @IBAction func deletePresed(_ sender: AnyObject) {
    PHPhotoLibrary.shared().performChanges({
      PHAssetChangeRequest.deleteAssets([self.photoAsset] as NSArray)
      }, completionHandler: nil)
  }
  
  // MARK: - Private helper methods
  fileprivate func displayImage() {
    guard let asset = photoAsset else { return }
    photoLocalIdentifier = asset.localIdentifier
    // Load a high quality image
    let options = PHImageRequestOptions()
    options.deliveryMode = .highQualityFormat
    options.isNetworkAccessAllowed = true
    
    PHImageManager.default().requestImage(for: asset,
                                          targetSize: targetSize,
                                          contentMode: .aspectFill,
                                          options: options)
    { image, _ in
      guard let image = image else { return }
      if self.photoLocalIdentifier == asset.localIdentifier {
        self.imageView.image = image
      }
    }
  }
  
}

extension AssetDetailViewController: PHPhotoLibraryChangeObserver {
  func photoLibraryDidChange(_ changeInstance: PHChange) {
    DispatchQueue.main.async {
      // Check that change is for this asset
      guard let details = changeInstance.changeDetails(for: self.photoAsset) else { return }
      if details.objectWasDeleted {
        _ = self.navigationController?.popViewController(animated: true)
        return
      }
      // Update the asset based on changes
      self.photoAsset = details.objectAfterChanges as! PHAsset
      // Redisplay the image if changed
      if details.assetContentChanged {
        self.displayImage()
      }
      self.favoriteButton.setTitle(
        self.photoAsset.isFavorite ? self.symbolFavorite : self.symbolUnfavorite,
        for: .normal)
    }
  }
}
