/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

class MasterViewController: UITableViewController {
  
  // Segues
  enum Segues: String {
    case SegueMasterToSlideShows
    case SegueAssetPicker
  }
  
  var detailViewController: AssetGridViewController? = nil
  var selectedAssets: SelectedAssets?
  
  fileprivate let sectionNames = ["","","Albums"]
  fileprivate var userAlbums: PHFetchResult<PHCollection>!
  fileprivate var userFavorites: PHFetchResult<PHAssetCollection>!
  
  override func viewDidLoad() {
    super.viewDidLoad()
    if selectedAssets == nil {
      selectedAssets = SelectedAssets()
    }
    PHPhotoLibrary.requestAuthorization { status in
      DispatchQueue.main.async {
        switch status {
        case .authorized:
          self.fetchCollections()
          self.tableView.reloadData()
        default:
          self.showNoAccessAlertAndCancel()
        }
      }
    }
    if let split = self.splitViewController {
      let controllers = split.viewControllers
      self.detailViewController = (controllers[controllers.count-1] as! UINavigationController).topViewController as? AssetGridViewController
    }
  }
  
  override func viewWillAppear(_ animated: Bool) {
    self.clearsSelectionOnViewWillAppear = self.splitViewController!.isCollapsed
    super.viewWillAppear(animated)
    tableView.reloadData()
  }
  
  // MARK: - Segues
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    if segue.identifier == Segues.SegueAssetPicker.rawValue {
      if let indexPath = self.tableView.indexPathForSelectedRow {
        let destination = (segue.destination as! UINavigationController).childViewControllers.first as! AssetGridViewController
        let cell = sender as! UITableViewCell
        destination.title = cell.textLabel?.text
        destination.selectedAssets = selectedAssets
        destination.delegate = self
        
        let options = PHFetchOptions()
        options.sortDescriptors =
          [NSSortDescriptor(key: "creationDate", ascending: true)]
        switch (indexPath.section) {
        case 0:
          // Selected
          destination.assetsFetchResults = nil
        case 1:
          if indexPath.row == 0 {
            // All Photos
            destination.assetsFetchResults  =
              PHAsset.fetchAssets(with: options) as? PHFetchResult<AnyObject>
          } else {
            // Favorites
            let favorites = userFavorites[indexPath.row - 1]
            destination.assetsFetchResults =
              PHAsset.fetchAssets(in: favorites,
                                  options: options) as? PHFetchResult<AnyObject>
          }
        case 2:
          // Albums
          let album = userAlbums[indexPath.row] as! PHAssetCollection
          destination.assetsFetchResults =
            PHAsset.fetchAssets(in: album,
                                options: options) as? PHFetchResult<AnyObject>
        default:
          break
        }
      }
    }
  }
  
  // MARK: - Private helper methods
  func fetchCollections() {
    userAlbums = PHCollectionList.fetchTopLevelUserCollections(with: nil)
    userFavorites = PHAssetCollection.fetchAssetCollections(
      with: .smartAlbum,
      subtype: .smartAlbumFavorites,
      options: nil)
  }
  
  func showNoAccessAlertAndCancel() {
    let alert = UIAlertController(title: "No Photo Permissions", message: "Please grant photo permissions in Settings", preferredStyle: .alert)
    alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
    alert.addAction(UIAlertAction(title: "Settings", style: .default, handler: { _ in
      UIApplication.shared.open(
        URL(string: UIApplicationOpenSettingsURLString)!,
        options: [:],
        completionHandler: nil)
      return
    }))
    
    self.present(alert, animated: true, completion: nil)
  }
}

// MARK: - Table View
extension MasterViewController {
  override func numberOfSections(in tableView: UITableView) -> Int {
    return sectionNames.count
  }
  
  override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    switch (section) {
    case 0:
      return 1
    case 1:
      return 1 + (userFavorites?.count ?? 0)
    case 2:
      return userAlbums?.count ?? 0
    default:
      return 0
    }
  }
  
  override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
    let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath)
    cell.textLabel!.text = ""
    
    switch(indexPath.section) {
    case 0:
      cell.textLabel!.text = "Selected"
    case 1:
      if indexPath.row == 0 {
        cell.textLabel!.text = "All Photos"
      } else {
        let favorite = userFavorites[indexPath.row - 1]
        cell.textLabel!.text = favorite.localizedTitle
      }
    case 2:
      let album = userAlbums[indexPath.row] as! PHAssetCollection
      cell.textLabel!.text = album.localizedTitle
    default:
      break
    }
    return cell
  }
}

extension MasterViewController: AssetPickerDelegate {
  func assetPickerDidCancel() {
    // Do nothing for now
  }
  
  func assetPickerDidFinishPickingAssets(_ selectedAssets: [PHAsset])  {
    // Create a new slideshow
    if selectedAssets.count > 0 {
      let identifiers = selectedAssets.map { asset in asset.localIdentifier }
      let slideshow = Slideshow(identifiers: identifiers)
      SlideshowManager.sharedInstance.addSlideshow(slideshow)
      // Clear out selections
      self.selectedAssets?.assets.removeAll()
    }
  }
}

