/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Foundation
import MultipeerConnectivity

class PeerToPeerConnectionManager: NSObject {
  static let sharedInstance = PeerToPeerConnectionManager()
  
  private var advertiserAssistant: MCAdvertiserAssistant?
  private let peerId = MCPeerID(displayName: UIDevice.current.name)
  private var session: MCSession?
  private let serviceType = "rw-fuse"
  fileprivate let archiveClassName = "Fuse.Slideshow"
  
  func setupPeerToPeerAdvertiser() {
    session = MCSession(peer: peerId, securityIdentity: nil, encryptionPreference: .none)
    session?.delegate = self
    advertiserAssistant = MCAdvertiserAssistant(
      serviceType: serviceType,
      discoveryInfo: nil,
      session: session!)
    advertiserAssistant?.start()
  }
  
  deinit {
    advertiserAssistant?.stop()
  }
  
  func isConnected() -> Bool {
    if let connectedPeers = session?.connectedPeers {
      return connectedPeers.count > 0
    }
    return false
  }
  
  func sendToAllPeers(_ slideshow: Slideshow) -> Bool {
    guard isConnected() else { return  false }
    let data = NSKeyedArchiver.archivedData(withRootObject: slideshow)
    do {
      try session?.send(data, toPeers: (session?.connectedPeers)!, with: .reliable)
      return true
    } catch {
      return false
    }
  }
  
  // MARK: - Private helper methods
  fileprivate func sendSlideshowToPeer(_ peerID: MCPeerID) {
    let slideshows = SlideshowManager.sharedInstance.slideshows
    if slideshows.count > 0 {
      // Pick a random slideshow to send to peers
      let slideshowIndex = Int(arc4random_uniform(UInt32(slideshows.count)))
      let data = NSKeyedArchiver.archivedData(withRootObject: slideshows[slideshowIndex])
      do {
        try session?.send(data, toPeers: [peerID], with: .reliable)
      } catch {
        // Do nothing
      }
    }
  }
}

// MARK: - MCSessionDelegate
extension PeerToPeerConnectionManager: MCSessionDelegate {
  func session(_ session: MCSession, didReceive data: Data, fromPeer peerID: MCPeerID) {
    NSKeyedUnarchiver.setClass(Slideshow.self, forClassName: archiveClassName)
    if let data = NSKeyedUnarchiver.unarchiveObject(with: data) {
      // Checking if incoming data is a slideshow 
      if let slideshowData = data as? Slideshow {
        DispatchQueue.main.async{
          // Update the slideshow's theme
          SlideshowManager.sharedInstance.updateSlideshow(slideshowData)
          NotificationCenter.default.post(
            name: .dataReceived, object: nil, userInfo: ["slideshowKey": slideshowData.getKey()])
        }
      } else {
        if let data = data as? [String:Any],
          let slideshowKey = data["slideshowKey"] as? String,
          let image = data["asset"] as? UIImage,
          let slideshow =
          SlideshowManager.sharedInstance.getSlideshow(slideshowKey) {
          DispatchQueue.main.async{
            // Add the image to the Photo Library then to the slideshow
            SlideshowManager.sharedInstance.add(image, toSlideshow: slideshow)
            NotificationCenter.default.post(
              name: .dataReceived, object: nil, userInfo: ["slideshowKey": slideshowKey])
          }
        }
      }
    }
  }
  func session(_ session: MCSession, didReceive stream: InputStream, withName streamName: String, fromPeer peerID: MCPeerID) {
    //
  }
  func session(_ session: MCSession, didStartReceivingResourceWithName resourceName: String, fromPeer peerID: MCPeerID, with progress: Progress) {
    //
  }
  func session(_ session: MCSession, didFinishReceivingResourceWithName resourceName: String, fromPeer peerID: MCPeerID, at localURL: URL, withError error: Error?) {
    //
  }
  func session(_ session: MCSession, peer peerID: MCPeerID, didChange state: MCSessionState) {
    if state == .connected {
      print("Connected to \(peerID.displayName)")
      sendSlideshowToPeer(peerID)
    }
  }
}

// MARK: - Notification
extension Notification.Name {
  static let dataReceived = Notification.Name("com.razeware.apps.Sliders:DataReceived")
}
