/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

class SlideshowPageViewController: UIPageViewController {
  
  // MARK: - Variables
  var slideshow: Slideshow?
  var timeInterval: TimeInterval = 3.0
  var numberOfTimesToLoop = 1
  
  fileprivate var assetsFetchResults: PHFetchResult<PHAsset>?
  private var loopCounter = 0
  fileprivate var currentIndex = 0
  private let storyboadIdentifier = "SlideshowDetailViewController"
  private var timer: Timer?
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    self.automaticallyAdjustsScrollViewInsets = false
    
    assetsFetchResults = slideshow?.fetchAssets()
    
    dataSource = self
    
    if let viewController = viewPhotoController(currentIndex) {
      let viewControllers = [viewController]
      setViewControllers(viewControllers,
                         direction: .forward,
                         animated: false,
                         completion: nil)
    }
    
    // Scroll automatically
    timer = Timer.scheduledTimer(withTimeInterval: timeInterval, repeats: true) { (timerIn) in
      self.advanceSlideshow()
    }
    
    NotificationCenter.default.addObserver(self, selector: #selector(updateSlideshow(notification:)), name: .dataReceived, object: nil)
  }
  
  deinit {
    NotificationCenter.default.removeObserver(self)
  }
  
  func updateSlideshow(notification: NSNotification) {
    if let slideshowInfo = notification.userInfo,
      let userInfoSlideshowKey = slideshowInfo["slideshowKey"] as? String {
      if slideshow?.getKey() == userInfoSlideshowKey {
        if let page = viewControllers?[0] as? SlideshowDetailViewController,
          let slideshowForKey = SlideshowManager.sharedInstance.getSlideshow(userInfoSlideshowKey) {
          slideshow = slideshowForKey
          // If the slideshow theme is changed
          page.photoTitle = slideshow?.getTitle()
          page.view.backgroundColor = slideshow?.getThemeBackgroundColor()
        }
      }
    }
  }
  
  // MARK: - Private helper methods
  func viewPhotoController(_ index: Int) -> SlideshowDetailViewController? {
    if let storyboard = storyboard,
      let page = storyboard.instantiateViewController(withIdentifier: storyboadIdentifier) as? SlideshowDetailViewController {
      page.photoAsset = assetsFetchResults![index]
      page.photoIndex = index
      page.photoTitle = slideshow?.getTitle()
      page.view.backgroundColor = slideshow?.getThemeBackgroundColor()
      return page
    }
    return nil
  }
  
  private func advanceSlideshow() {
    guard (assetsFetchResults?.count)! > 0 else { return }
    if assetsFetchResults?.count == currentIndex + 1 && numberOfTimesToLoop == loopCounter + 1 {
      timer?.invalidate()
      return
    }
    if assetsFetchResults?.count == currentIndex + 1 {
      loopCounter = loopCounter + 1
      currentIndex = 0
    } else {
      currentIndex = currentIndex + 1
    }
    if let nextViewController = viewPhotoController(currentIndex) {
      let viewControllers = [nextViewController]
      setViewControllers(viewControllers,
                         direction: .forward,
                         animated: true,
                         completion: nil)
    }
  }
}

// MARK: - UIPageViewControllerDataSource
extension SlideshowPageViewController: UIPageViewControllerDataSource {
  func pageViewController(_ pageViewController: UIPageViewController,
                          viewControllerBefore viewController: UIViewController) -> UIViewController? {
    
    if let viewController = viewController as? SlideshowDetailViewController {
      var index = viewController.photoIndex
      guard index != NSNotFound && index != 0 else { return nil }
      index = index! - 1
      return viewPhotoController(index!)
    }
    return nil
  }
  
  func pageViewController(_ pageViewController: UIPageViewController,
                          viewControllerAfter viewController: UIViewController) -> UIViewController? {
    
    if let viewController = viewController as? SlideshowDetailViewController {
      var index = viewController.photoIndex
      guard index != NSNotFound else { return nil }
      index = index! + 1
      let assetCount = assetsFetchResults?.count ?? 0
      guard index != assetCount else {return nil}
      return viewPhotoController(index!)
    }
    return nil
  }
  
  // MARK: - UIPageControl
  func presentationCount(for pageViewController: UIPageViewController) -> Int {
    return assetsFetchResults?.count ?? 0
  }
  
  func presentationIndex(for pageViewController: UIPageViewController) -> Int {
    return currentIndex
  }
}
