/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

class SlideshowsViewController: UICollectionViewController {
  
  @IBOutlet weak var emptySlideshowView: UIView!
  
  // MARK: - Variables
  fileprivate let reuseIdentifier = "SlideshowCell"
  
  // Segues
  enum Segues: String {
    case SegueSlideshowsToAssetPicker
    case SegueSlideshowsToSlideshowDetail
  }
  
  fileprivate var slideshows: [Slideshow] = [] {
    didSet {
      emptySlideshowView.isHidden = slideshows.count > 0
    }
  }
  private var assetsFetchResults: PHFetchResult<PHAsset>?
  fileprivate var slideshowAssets: [PHAsset]?
  fileprivate var thumbnailSize = CGSize.zero
  fileprivate let imageManager = PHCachingImageManager()
  fileprivate var cachingIndexes: [IndexPath] = []
  fileprivate var lastCacheFrameCenter: CGFloat = 0
  private var cacheQueue = DispatchQueue(label: "cache_queue")
  
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    self.navigationItem.leftBarButtonItem = nil
    self.splitViewController?.preferredDisplayMode = .primaryHidden
    
    resetCache()
    
    // Notification observer
    NotificationCenter.default.addObserver(self, selector: #selector(noteSlideshow), name: .dataReceived, object: nil)
    
    // Set up gesture for sending slideshow
    let gestureRecognizer = UILongPressGestureRecognizer(target: self, action: #selector(handlePress(_:)))
    collectionView?.addGestureRecognizer(gestureRecognizer)
  }
  
  deinit {
    NotificationCenter.default.removeObserver(self)
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    
    // Determine the size of the thumbnails to request from the PHCachingImageManager
    let scale = UIScreen.main.scale
    let cellSize = (collectionViewLayout as! UICollectionViewFlowLayout).itemSize
    thumbnailSize = CGSize(width: cellSize.width * scale, height: cellSize.height * scale)
    
    // Load and display slideshows
    loadSlideshows()
  }
  
  // MARK: - Navigation
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    if segue.identifier == Segues.SegueSlideshowsToSlideshowDetail.rawValue {
      let destination = segue.destination as! SlideshowPageViewController
      let indexPath = collectionView!.indexPath(for: sender as! GridViewCell)!
      let slideshow = slideshows[indexPath.item] 
      destination.slideshow = slideshow
    } else if segue.identifier == Segues.SegueSlideshowsToAssetPicker.rawValue {
      let destination = (segue.destination as! UINavigationController).childViewControllers.first as! AssetGridViewController
      destination.selectedAssets = SelectedAssets()
    }
  }
  
  // MARK: - UIScrollViewDelegate
  override func scrollViewDidScroll(_ scrollView: UIScrollView) {
    cacheQueue.async {
      self.updateCache()
    }
  }
    
  // MARK: - Helpers
  func noteSlideshow() {
    // Load and display slideshows
    loadSlideshows()
  }
  
  func handlePress(_ sender: UILongPressGestureRecognizer) {
    if sender.state == .ended {
      if let cell = UIScreen.main.focusedView as? GridViewCell,
        let indexPath = collectionView?.indexPath(for: cell) {
        if PeerToPeerConnectionManager.sharedInstance.sendToAllPeers(slideshows[indexPath.item]) {
          let alert = UIAlertController(title: "Share",
                                        message: "Slideshow shared successfully.",
                                        preferredStyle: .alert)
          alert.addAction(UIAlertAction(title: "OK",
                                        style: .cancel, handler: nil))
          present(alert, animated: true, completion: nil)
        }
      }
    }
  }
  
  // MARK: - Private helper methods
  private func slideshowAssetForIdentifier(_ identifier: String) -> PHAsset? {
    if let fetchResults = assetsFetchResults {
      for i in 0..<fetchResults.count {
        if fetchResults.object(at: i).localIdentifier == identifier {
          return fetchResults.object(at: i)
        }
      }
    }
    return nil
  }
  
  private func loadSlideshows() {
    // Get stored slideshows
    slideshows = SlideshowManager.sharedInstance.slideshows
    // Fetch thumbnails for slideshows, thumbnails are the first asset in the slideshow
    if slideshows.count > 0 {
      let thumbnailIdentifiers = slideshows.map { slideshow in slideshow.thumbnailIdentifier()! }
      assetsFetchResults = PHAsset.fetchAssets(withLocalIdentifiers: thumbnailIdentifiers, options: nil)
      slideshowAssets = slideshows.map { slideshow in slideshowAssetForIdentifier(slideshow.thumbnailIdentifier()!)!
      }
    }
    // Display data in collection view
    collectionView!.reloadData()
  }
  
}

// MARK: - Caching
extension SlideshowsViewController {
  func resetCache() {
    imageManager.stopCachingImagesForAllAssets()
    cachingIndexes.removeAll(keepingCapacity: true)
    lastCacheFrameCenter = 0
  }
  
  func updateCache() {
    let currentFrameCenter = collectionView!.bounds.midY
    if abs(currentFrameCenter - lastCacheFrameCenter) <
      collectionView!.bounds.height / 3 {
      return
    }
    lastCacheFrameCenter = currentFrameCenter
    let numOffscreenAssetsToCache = 60
    
    var visibleIndexes = collectionView!.indexPathsForVisibleItems
    
    _ = visibleIndexes.sorted { a, b in
      (a as NSIndexPath).item < (b as NSIndexPath).item
    }
    if visibleIndexes.count == 0 {
      return
    }
    
    let totalItemCount = slideshows.count
    let lastItemToCache = min(totalItemCount,visibleIndexes[visibleIndexes.count-1].item + numOffscreenAssetsToCache/2)
    let firstItemToCache = max(0, visibleIndexes[0].item - numOffscreenAssetsToCache/2)
    
    let options = PHImageRequestOptions()
    options.isNetworkAccessAllowed = true
    options.resizeMode = .fast
    
    var indexesToStopCaching: [IndexPath] = []
    cachingIndexes = cachingIndexes.filter { index in
      if (index as NSIndexPath).item < firstItemToCache || (index as NSIndexPath).item > lastItemToCache {
        indexesToStopCaching.append(index)
        return false
      }
      return true
    }
    imageManager.stopCachingImages(for: assetsAtIndexPaths(indexesToStopCaching),
                                   targetSize: thumbnailSize,
                                   contentMode: .aspectFill,
                                   options: options)
    
    var indexesToStartCaching: [IndexPath] = []
    for i in firstItemToCache..<lastItemToCache {
      let indexPath = IndexPath(item: i, section: 0)
      if !cachingIndexes.contains(indexPath) {
        indexesToStartCaching.append(indexPath)
      }
    }
    cachingIndexes += indexesToStartCaching
    imageManager.startCachingImages(
      for: assetsAtIndexPaths(indexesToStartCaching),
      targetSize: thumbnailSize, contentMode: .aspectFill,
      options: options)
  }
  
  private func assetsAtIndexPaths(_ indexPaths:[IndexPath]) -> [PHAsset] {
    return indexPaths.map { indexPath in
      return slideshowAssets![indexPath.item]
    }
  }
}

// MARK: - UICollectionViewDelegate, UICollectionViewDataSource
extension SlideshowsViewController {
  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return slideshows.count
  }
  
  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: reuseIdentifier, for: indexPath) as! GridViewCell
    
    // Configure the cell
    let asset = slideshowAssets![indexPath.item]
    let options = PHImageRequestOptions()
    options.isNetworkAccessAllowed = true
    
    cell.representedAssetIdentifier = asset.localIdentifier
    imageManager.requestImage(for: asset,
                              targetSize: thumbnailSize,
                              contentMode: .aspectFill,
                              options: options)
    { image, _ in
      if cell.representedAssetIdentifier == asset.localIdentifier {
        cell.thumbnailImage = image
        // Update slideshow image
        self.slideshows[indexPath.item].setImage(image)
      }
    }
    
    cell.label.text = slideshows[indexPath.item].getTitle()
    
    return cell
  }
  
}
