/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit
import Photos

protocol AssetPickerDelegate {
  func assetPickerDidFinishPickingAssets(_ selectedAssets: [PHAsset])
  func assetPickerDidCancel()
}

class AssetGridViewController: UICollectionViewController {
  
  // MARK: - Variable enums
  
  // Types for photo selections
  enum AssetSelection: String {
    case selected
    case allPhotos
    case favorites
    case albums
  }
  
  // Segues
  enum Segues: String {
    case SegueDetailToSlideShows
    case SegueAssetDetail
  }
  
  // MARK: - Variables
  public var assetsFetchResults: PHFetchResult<AnyObject>?
  var selectedAssets: SelectedAssets!
  var selectionMode: AssetSelection!
  var delegate: AssetPickerDelegate?
  
  fileprivate let reuseIdentifier = "AssetCell"
  private let slideshowsSegue = "showSlideshows"
  
  fileprivate var thumbnailSize = CGSize.zero
  fileprivate let imageManager = PHCachingImageManager()
  fileprivate var cachingIndexes: [IndexPath] = []
  fileprivate var lastCacheFrameCenter: CGFloat = 0
  private var cacheQueue = DispatchQueue(label: "cache_queue")
  
  fileprivate var editMode = false
  
  private var doneButton: UIBarButtonItem!
  private var editModeEditButton: UIBarButtonItem!
  private var editModeDoneButton: UIBarButtonItem!
  
  override func viewDidLoad() {
    super.viewDidLoad()
    self.splitViewController?.preferredDisplayMode = .allVisible
    collectionView!.allowsMultipleSelection = true
    resetCache()
    PHPhotoLibrary.shared().register(self)
    doneButton = UIBarButtonItem(title: "Create Slideshow", style: .done, target: self, action: #selector(donePressed(_:)))
    editModeEditButton = UIBarButtonItem(title: "Select Mode", style: .plain, target: self, action: #selector(editModeToggled(_:)))
    editModeDoneButton = UIBarButtonItem(title: "Edit Mode", style: .done, target: self, action: #selector(editModeToggled(_:)))
  }
  
  deinit {
    PHPhotoLibrary.shared().unregisterChangeObserver(self)
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    
    // Determine the size of the thumbnails to request from the PHCachingImageManager
    let scale = UIScreen.main.scale
    let cellSize = (collectionViewLayout as! UICollectionViewFlowLayout).itemSize
    thumbnailSize = CGSize(width: cellSize.width * scale, height: cellSize.height * scale)
    
    collectionView!.reloadData()
    updateSelectedItems()
    updateEditButton()
  }
  
  override func viewDidAppear(_ animated: Bool) {
    super.viewDidAppear(animated)
    updateCache()
  }
  
  override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
    super.viewWillTransition(to: size, with: coordinator)
    collectionView!.reloadData()
    updateSelectedItems()
  }
  
  // MARK: - Navigation
  override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
    if segue.identifier == Segues.SegueDetailToSlideShows.rawValue {
      delegate?.assetPickerDidFinishPickingAssets(selectedAssets.assets)
    } else if segue.identifier == Segues.SegueAssetDetail.rawValue {
      let destination = segue.destination as! AssetDetailViewController
      let indexPath = collectionView!.indexPath(for: sender as! GridViewCell)!
      destination.photoAsset = currentAsset(at: indexPath.item)
    }
  }
  
  override func shouldPerformSegue(withIdentifier identifier: String, sender: Any?) -> Bool {
    // Don't view details assets unless in edit mode
    if identifier == Segues.SegueAssetDetail.rawValue && !editMode {
      return false
    }
    return true
  }
  
  // MARK: - Button handlers
  func donePressed(_ sender: UIBarButtonItem) {
    performSegue(withIdentifier: Segues.SegueDetailToSlideShows.rawValue, sender: sender)
  }
  
  func editModeToggled(_ sender: UIBarButtonItem) {
    editMode = !editMode
    updateEditButton()
  }
  
  // MARK: - UIScrollViewDelegate
  override func scrollViewDidScroll(_ scrollView: UIScrollView) {
    cacheQueue.async {
      self.updateCache()
    }
  }
  
  // MARK: - Private helper methods
  fileprivate func currentAsset(at index:NSInteger) -> PHAsset {
    if let fetchResult = assetsFetchResults {
      return fetchResult[index] as! PHAsset
    } else {
      return selectedAssets.assets[index]
    }
  }

  private func updateSelectedItems() {
    guard selectedAssets != nil else { return }
    // Select the selected items
    if let fetchResult = assetsFetchResults {
      for asset in selectedAssets.assets {
        let index = fetchResult.index(of: asset)
        if index != NSNotFound {
          let indexPath = IndexPath(item: index, section: 0)
          collectionView!.selectItem(at: indexPath,
                                     animated: false, scrollPosition: UICollectionViewScrollPosition())
        }
      }
    } else {
      for i in 0..<selectedAssets.assets.count {
        let indexPath = IndexPath(item: i, section: 0)
        collectionView!.selectItem(at: indexPath,
                                   animated: false, scrollPosition: UICollectionViewScrollPosition())
      }
    }
    updateDoneButton()
  }
  
  fileprivate func updateDoneButton() {
    guard selectedAssets != nil else { return }
    // Add a done button when there are selected assets
    if selectedAssets.assets.count > 0 {
      self.navigationItem.rightBarButtonItem = doneButton
    } else {
      self.navigationItem.rightBarButtonItem = nil
    }
  }
  
  private func updateEditButton() {
    var assetCount = 0
    if let fetchResult = assetsFetchResults {
      assetCount = fetchResult.count
    } else if selectedAssets != nil {
      assetCount = selectedAssets.assets.count
    }
    if assetCount > 0 {
      self.navigationItem.leftBarButtonItem = editMode ? editModeDoneButton : editModeEditButton
    } else {
      self.navigationItem.leftBarButtonItem = nil
    }
  }
  
}

// MARK: - Caching
extension AssetGridViewController {
  func resetCache() {
    imageManager.stopCachingImagesForAllAssets()
    cachingIndexes.removeAll(keepingCapacity: true)
    lastCacheFrameCenter = 0
  }
  
  func updateCache() {
    let currentFrameCenter = collectionView!.bounds.midY
    if abs(currentFrameCenter - lastCacheFrameCenter) <
      collectionView!.bounds.height / 3 {
      return
    }
    lastCacheFrameCenter = currentFrameCenter
    
    var visibleIndexes = collectionView!.indexPathsForVisibleItems
    _ = visibleIndexes.sorted { a, b in
      a.item < b.item
    }
    if visibleIndexes.count == 0 {
      return
    }
    
    var totalItemCount = selectedAssets.assets.count
    if let fetchResults = assetsFetchResults {
      totalItemCount = fetchResults.count
    }
    let numOffscreenAssetsToCache = 60
    let lastItemToCache = min(totalItemCount,visibleIndexes[visibleIndexes.count-1].item + numOffscreenAssetsToCache/2)
    let firstItemToCache = max(0, visibleIndexes[0].item - numOffscreenAssetsToCache/2)
    
    let options = PHImageRequestOptions()
    options.isNetworkAccessAllowed = true
    options.resizeMode = .fast
    
    var indexesToStopCaching: [IndexPath] = []
    cachingIndexes = cachingIndexes.filter { index in
      if index.item < firstItemToCache || index.item > lastItemToCache {
        indexesToStopCaching.append(index)
        return false
      }
      return true
    }
    imageManager.stopCachingImages(for: assetsAtIndexPaths(indexesToStopCaching),
                                   targetSize: thumbnailSize,
                                   contentMode: .aspectFill,
                                   options: options)
    
    var indexesToStartCaching: [IndexPath] = []
    for i in firstItemToCache..<lastItemToCache {
      let indexPath = IndexPath(item: i, section: 0)
      if !cachingIndexes.contains(indexPath) {
        indexesToStartCaching.append(indexPath)
      }
    }
    cachingIndexes += indexesToStartCaching
    imageManager.startCachingImages(
      for: assetsAtIndexPaths(indexesToStartCaching),
      targetSize: thumbnailSize, contentMode: .aspectFill,
      options: options)
  }
  
  private func assetsAtIndexPaths(_ indexPaths:[IndexPath]) -> [PHAsset] {
    return indexPaths.map { indexPath in
      return self.currentAsset(at: indexPath.item)
    }
  }
}

// MARK: - UICollectionViewDelegate, UICollectionViewDataSource
extension AssetGridViewController {
  override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    if editMode { return }
    // Update selected Assets
    let asset = currentAsset(at: indexPath.item)
    selectedAssets.assets.append(asset)
    let cell = collectionView.cellForItem(at: indexPath) as! GridViewCell
    cell.checkMarkSelected(true)
    updateDoneButton()
  }
  
  override func collectionView(_ collectionView: UICollectionView, didDeselectItemAt indexPath: IndexPath) {
    if editMode { return }
    // Update de-selected Assets
    let assetToDelete = currentAsset(at: indexPath.item)
    selectedAssets.assets = selectedAssets.assets.filter { asset in
      !(asset == assetToDelete)
    }
    if assetsFetchResults == nil {
      collectionView.deleteItems(at: [indexPath])
    }
    let cell = collectionView.cellForItem(at: indexPath) as! GridViewCell
    cell.checkMarkSelected(false)
    updateDoneButton()
  }
  
  override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int  {
    if let fetchResult = assetsFetchResults {
      return fetchResult.count
    } else if selectedAssets != nil {
      return selectedAssets.assets.count
    }
    return 0
  }
  
  override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCell(withReuseIdentifier: reuseIdentifier, for: indexPath) as! GridViewCell
    
    // Populate Cell
    let asset = currentAsset(at: indexPath.item)
    
    let options = PHImageRequestOptions()
    options.isNetworkAccessAllowed = true
    
    cell.representedAssetIdentifier = asset.localIdentifier
    imageManager.requestImage(for: asset,
                              targetSize: thumbnailSize,
                              contentMode: .aspectFill,
                              options: options)
    { image, _ in
      if cell.representedAssetIdentifier == asset.localIdentifier {
        cell.thumbnailImage = image
      }
    }
    
    return cell
  }
}

// MARK: - UICollectionViewDelegateFlowLayout
extension AssetGridViewController: UICollectionViewDelegateFlowLayout {
  func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
    var thumbsPerRow: Int
    switch collectionView.bounds.size.width {
    case 0..<400:
      thumbsPerRow = 4
    case 400..<600:
      thumbsPerRow = 5
    case 600..<800:
      thumbsPerRow = 6
    case 800..<1200:
      thumbsPerRow = 7
    default:
      thumbsPerRow = 4
    }
    let width = collectionView.bounds.size.width / CGFloat(thumbsPerRow)
    return CGSize(width: width,height: width)
  }
}

// MARK: - PHPhotoLibraryChangeObserver
extension AssetGridViewController: PHPhotoLibraryChangeObserver {
  func photoLibraryDidChange(_ changeInstance: PHChange)  {
    guard assetsFetchResults != nil else { return }
    // Respond to changes
    DispatchQueue.main.async {
      if let fetchResult = self.assetsFetchResults as? PHFetchResult<PHObject>,
        let collectionChanges =
        changeInstance.changeDetails(for: fetchResult)
      {
        self.assetsFetchResults = collectionChanges.fetchResultAfterChanges as? PHFetchResult<AnyObject>
        if collectionChanges.hasMoves ||
          !collectionChanges.hasIncrementalChanges {
          self.collectionView!.reloadData()
        } else {
          // perform incremental updates
          self.collectionView!.performBatchUpdates({
            if let removedIndexes = collectionChanges.removedIndexes {
              if removedIndexes.count > 0 {
                self.collectionView!.deleteItems(
                  at: self.indexPathsFromIndexSet(removedIndexes, section: 0))
              }
            }
            if let insertedIndexes = collectionChanges.insertedIndexes {
              if insertedIndexes.count > 0 {
                self.collectionView!.insertItems(
                  at: self.indexPathsFromIndexSet(insertedIndexes, section: 0))
              }
            }
            if let changedIndexes = collectionChanges.changedIndexes {
              if changedIndexes.count > 0 {
                self.collectionView!.reloadItems(
                  at: self.indexPathsFromIndexSet(changedIndexes, section: 0))
              }
            }
            }, completion: nil)
        }
      }
    }
  }
  
  func indexPathsFromIndexSet(_ indexSet:IndexSet, section:Int) -> [IndexPath] {
    var indexPaths: [IndexPath] = []
    for (i, _) in (indexSet as IndexSet).enumerated() {
      indexPaths.append(IndexPath(item: i, section: section))
    }
    return indexPaths
  }
}

