/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Foundation
import Photos

class Slideshow: NSObject {
  
  enum SlideshowFields: String {
    case Key
    case Identifiers
    case Title
    case Theme
    case Image
  }
  
  var record: [String:Any?]
  
  override init() {
    record = [
      SlideshowFields.Key.rawValue: UUID.init().uuidString,
      SlideshowFields.Identifiers.rawValue: [],
      SlideshowFields.Title.rawValue: "",
      SlideshowFields.Theme.rawValue: "",
      SlideshowFields.Image.rawValue: nil
    ]
  }
  
  init(identifiers: [String]) {
    record = [
      SlideshowFields.Key.rawValue: UUID.init().uuidString,
      SlideshowFields.Identifiers.rawValue: identifiers,
      SlideshowFields.Title.rawValue: "",
      SlideshowFields.Theme.rawValue: "",
      SlideshowFields.Image.rawValue: nil
    ]
  }
  
  // MARK: - NSCoder
  required init?(coder aDecoder: NSCoder) {
    guard
      let decodedKey = aDecoder.decodeObject(forKey: SlideshowFields.Key.rawValue) as? String,
      let decodedIdentifiers = aDecoder.decodeObject(forKey: SlideshowFields.Identifiers.rawValue) else {
        fatalError("Slideshow object could not be created")
    }
    let decodedTitle = aDecoder.decodeObject(forKey: SlideshowFields.Title.rawValue) as? String ?? ""
    let decodedTheme = aDecoder.decodeObject(forKey: SlideshowFields.Theme.rawValue) as? String ?? ""
    var decodedImage: UIImage?
    if let imageData = aDecoder.decodeObject(forKey: SlideshowFields.Image.rawValue) as? Data {
      decodedImage = UIImage(data: imageData)
    }
    record = [
      SlideshowFields.Key.rawValue: decodedKey,
      SlideshowFields.Identifiers.rawValue: decodedIdentifiers,
      SlideshowFields.Title.rawValue: decodedTitle,
      SlideshowFields.Theme.rawValue: decodedTheme,
      SlideshowFields.Image.rawValue: decodedImage
    ]
  }
  
  func encodeWithCoder(_ aCoder: NSCoder) {
    aCoder.encode(record[SlideshowFields.Key.rawValue] as? String,
                  forKey: SlideshowFields.Key.rawValue)
    aCoder.encode(record[SlideshowFields.Identifiers.rawValue] as? [String],
                  forKey: SlideshowFields.Identifiers.rawValue)
    aCoder.encode(record[SlideshowFields.Title.rawValue] as? String,
                  forKey: SlideshowFields.Title.rawValue)
    aCoder.encode(record[SlideshowFields.Theme.rawValue] as? String,
                  forKey: SlideshowFields.Theme.rawValue)
    if let image = record[SlideshowFields.Image.rawValue] as? UIImage,
      let imageData = UIImagePNGRepresentation(image) {
      aCoder.encode(imageData, forKey: SlideshowFields.Image.rawValue)
    }
  }
  
  // MARK: - Helpers
  func fetchAssets() -> PHFetchResult<PHAsset>? {
    guard let identifiers = record[SlideshowFields.Identifiers.rawValue] as? [String] else { return nil }
    if identifiers.count > 0 {
      let options = PHFetchOptions()
      options.sortDescriptors =
        [NSSortDescriptor(key: "creationDate", ascending: true)]
      return PHAsset.fetchAssets(withLocalIdentifiers: identifiers, options: options)
    }
    return nil
  }
  
  func addAsset(_ identifier: String) {
    if var identifiers = record[SlideshowFields.Identifiers.rawValue] as? [String] {
      identifiers.append(identifier)
      record[SlideshowFields.Identifiers.rawValue] = identifiers
    } else {
      record[SlideshowFields.Identifiers.rawValue] = [identifier]
    }
  }
  
  func thumbnailIdentifier() -> String? {
    guard let identifiers = record[SlideshowFields.Identifiers.rawValue] as? [String] else { return nil }
    if identifiers.count > 0 {
      return identifiers[0]
    }
    return nil
  }
  
  func getThemeBackgroundColor() -> UIColor {
    let theme = getTheme()
    switch theme {
    case "red":
      return UIColor.red
    case "green":
      return UIColor.green
    case "blue":
      return UIColor.blue
    default:
      return UIColor.black
    }
  }
  
  func getKey() -> String {
    return record[SlideshowFields.Key.rawValue] as! String
  }
  
  func setTitle(_ title: String) {
    record[SlideshowFields.Title.rawValue] = title
  }
  
  func getTitle() -> String {
    return record[SlideshowFields.Title.rawValue] as! String
  }
  
  func setTheme(_ title: String) {
    record[SlideshowFields.Theme.rawValue] = title
  }
  
  func getTheme() -> String {
    return record[SlideshowFields.Theme.rawValue] as! String
  }
  
  func setImage(_ image: UIImage?) {
    if image != nil {
      record[SlideshowFields.Image.rawValue] = image
    }
  }
  
  func getImage() -> UIImage? {
    return record[SlideshowFields.Image.rawValue] as? UIImage
  }
  
  func clearImage() {
    record[SlideshowFields.Image.rawValue] = nil
  }
}
